#!/usr/bin/perl -w
#
# Usage: genPutUsers begin-userid num-users
#
# Note: This script requires the executable PutMultipleFiles
#       to be located in the current directory.
#
# Script to generate x number of user accounts in xml format
# and place them in the correct Provisioning directories.
#
# The starting user name must be an integer and the subseqent
# user names will increment upwards from there.
#
# A file called tplate will be used as the source for
# the contents of the generated files. Tplate should be a valid
# xml user file. The user name tag will be replaced with the
# correct tag for each user.
#
# This script will generate all files first in the current directory
# and then place them into the provisioning directories.
#
# Copyright 2000 by Wendy Breu 
#
#
# $Id: genPutUsers.pl,v 1.1 2000/10/03 22:16:46 marcl Exp $
#
#
###############################################################################
# display usage message
#
if (@ARGV != 2 ) {
  die "Usage:  $0  begin-userid num-users\n";
}

# verbose mode
$verbose = 0;

# parse arguments
$idbase = $ARGV[0];   # begin userid
$count = $ARGV[1];    # generate $count users

# set template file:
$template = "./tplate";      # this is the template file we want to clone

$id = $idbase;

# using $template file, generate $idbase+n.xml file in current directory
sub cloneAccount {

   # assume <TEMPLATE> and <OUTFILE> are already opened:
   while (<TEMPLATE>)
   {
       $line = $_;
       chomp($line);
       if (/(^<user><name>)/) {
          $newline = "<user><name>$id</name>";
          print OUTFILE "$newline\n"; 
          # printf( "newline = $newline\n" );
          next;
       }
       # ignore the rest of xml data, just output to OUTFILE:
       print OUTFILE "$line\n";
   }
}

# set up file handles to generate new user xml files:
sub prepareData {
   $outfile = "$id";    # file name = userAccount#
   if (-e $outfile )
   {
      rmdir $outfile;
   }

   # first, fix up the file content   
   open( TEMPLATE, "<" . $template) || die "Could not open $template\n";
   open(OUTFILE, ">$outfile");
      print "generating $outfile...\n" if $verbose;
      cloneAccount();
   close( OUTFILE );
   close( TEMPLATE );
}

# main prog
sub main {
   # loop $count times
   $i = $count;
   while ($i) {
       prepareData();     
       $id++;
       $i--;
   }   
   # call FileDataStoreWrapper to put the files in the hash bins:-
   # usage: <#users -> this determines the # of hash bin buckets, 1000==> 6>
   #        <provisioning root directory, e.g. /usr/local/vocal/prov....>
   #        <target directory name, e.g. Accounts>
   #        <begin User number, e.g. 5000>
   #        <# Users to generate: do it in batches, or else it'll take forever!>
   @args = ("./PutMultipleFiles", "1000", "/usr/local/vocal/provisioning_data", "Accounts", $idbase, $count);
   system( @args ) == 0 
      or die "system @args failed: $?";

   $id = $idbase;
   $i = $count;
   while ($i) {
       $outfile = "$id";    # file name = userAccount#
       unlink ${outfile};
       $id++;
       $i--;
   }

}

main();

## end of program
