/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSRouteInterface_cxx_Version = 
"$Id: LSRouteInterface.cxx,v 1.2 2000/12/18 23:48:48 bko Exp $";


#include <unistd.h>
#include <stdlib.h> 
#include <strstream.h>
#include "LSRouteInterface.hxx"
#include "TcpClientSocket.hxx"
#include "NetworkAddress.h"
#include "VNetworkException.hxx"
#include "cpLog.h"

LSRouteInterface* LSRouteInterface::_instance = 0;

LSRouteInterface&
LSRouteInterface::instance()
{
    if (!_instance)
    {
        _instance = new LSRouteInterface();
        cpLog(LOG_INFO, "Started LSRouteInterface");
    }
    return (*_instance);
}

void 
LSRouteInterface::destroy(void)
{
    delete LSRouteInterface::_instance;
    LSRouteInterface::_instance = 0;
}

LSRouteInterface::LSRouteInterface()
{
    ///Get the list of LSs in the system
    _listOfLS.push_back("ganesh:8000"); 
#ifdef __linux__
    if (atexit(LSRouteInterface::destroy) < 0)
    {
        cpLog(LOG_ALERT, "Failed to register with atexit()");
    };
#endif
    _connection = new Connection();
    connect();
}

void
LSRouteInterface::connect()
{
    if(_connection->isLive()) return;

    for(ListOfLS::iterator itr = _listOfLS.begin();
                  itr != _listOfLS.end(); itr++)
    {
        try
        {
            NetworkAddress na((*itr));
            TcpClientSocket cl(na, false);
            Connection conn;
            cl.connect();
            _connection = new Connection(cl.getConn());
            cpLog(LOG_INFO, "LSRouteInterface connected to (%s)", (*itr).c_str());
            break;
        }
        catch(VNetworkException& e)
        {
            cpLog(LOG_INFO, "Failed to connect to server (%s), trying next..", (*itr).c_str());
        }
    }
}

LSRouteInterface::~LSRouteInterface()
{
}

void
LSRouteInterface::processRequest() throw (VNetworkException&)
{
    try
    {
        //USe _connection
        //If connection is bad it will throw exception, try reconnect
    }
    catch(VNetworkException& e)
    {
        cpLog(LOG_ALERT, "Connecton error on (%s), reason (%s) retrying ...",
                          _connection->getDescription().c_str(),
                          e.getDescription().c_str());
        connect();
    }
}

