
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const Advertiser_cxx_Version = 
"$Id: Advertiser.cxx,v 1.12 2001/03/20 22:41:04 kevinmc Exp $";

#include "Advertiser.hxx"
#include "LSSessionManager.hxx"
#include "LSPeerSession.hxx"
#include "TRIBAgent.hxx"
#include "cpLog.h"


Advertiser::Advertiser()
{

}


Advertiser::~Advertiser( )
{
}

void
Advertiser::process()  
{
    cpLog(LOG_DEBUG, "Advertiser::process");
    LSSessionManager& sessionMgr = LSSessionManager::instance();
    const ProtoCapabilityList& pList = sessionMgr.getProtoCapabilityList();
    const AddrFamilyList& aList = sessionMgr.getAddrFamilyList();
    TRIBAgent& tribAgent = TRIBAgent::instance();
    tribAgent.readLock();
    sessionMgr.lockSessionList();
    const LSPeerList& peerList = sessionMgr.getSessionList();
    for (LSPeerList::const_iterator itr = peerList.begin();
                                    itr != peerList.end(); itr++)
    {
        if ((*itr)->isEstablished())
        {
            if (!(*itr)->isPeerInternal())
            {
                //ok to process session since it is established and external
                //however if the adjTribOut does not exist, ok to ignore
                const TRIPItadKey& key = (*itr)->getPeerItadKey();
                try
                {
                    Sptr<TRIPTrieCache> tribOut = tribAgent.getTribOut(key);
                    tribOut->writeLock();
                    if (tribOut->isUpdated())
                    {
                        tribOut->setUpdated(false);
                        for (AddrFamilyList::const_iterator aItr = aList.begin();
                                    aItr != aList.end(); aItr++)
                        {
                            for (ProtoCapabilityList::const_iterator pItr = pList.begin();
                                            pItr != pList.end(); pItr++)
                            {
                                TRIPProtoKey key((*pItr), (*aItr));
                                Sptr <TRIPRouteSet> tribOutRouteSet = 
                                                    tribOut->getSortedRouteSet(key); 
                                if (tribOutRouteSet->size() == 0)
                                {
                                    continue;
                                }
                                Sptr <UpdateMsgList> msgList =
                                                    getMsgList(tribOutRouteSet);
                                if ((msgList.getPtr()) && (msgList->size() > 0))
                                {
                                    (*itr)->sendUpdateMsgList(msgList);
                                }
                            }
                        }
                    }
                    tribOut->unlock();
                }
                catch (VMissingDataException& e)
                {
                    cpLog(LOG_INFO, "(%d, %u):non existant Tribout", 
                                    key.getItad(), key.getTripId().getId() ); 
                }
            }
        }
    }
    sessionMgr.unlockSessionList();
    tribAgent.unlock();
}

Sptr <UpdateMsgList>
Advertiser::getMsgList(Sptr <TRIPRouteSet> srcSet)
{
    if (!(srcSet.getPtr()))
    {
        return 0;
    }
    if (srcSet->size() == 0)
    {
        return 0;
    }
    Sptr <UpdateMsgList> retList = new UpdateMsgList();
    u_int32_t prevTranId = 0;
    bool msgToSend = false;
    bool withdrawn = false;
    Sptr<TRIPUpdateMsg> uMsg;
    Sptr<TRIPWithdrawnRoute> wRoute;
    Sptr<TRIPReachableRoute> rRoute;
    Sptr<TRIPRouteElement> route;
    if ( cpLogGetPriority() >= LOG_DEBUG )
    {
        for(TRIPRouteSet::iterator iitr = srcSet->begin();
                                                iitr != srcSet->end(); iitr++)
        {
            (*iitr)->prettyDump(cerr);
        }
    }
    for(TRIPRouteSet::const_iterator itr = srcSet->begin();
                                    itr != srcSet->end(); itr++)
    {
        if (!((*itr)->isChanged(T_UPDATE_BIT)))
        {
            continue;
        }
        withdrawn = false;
        if ((*itr)->isWithdrawn())
        {
            withdrawn = true;
        }

        if (((*itr)->getTransactionId() == prevTranId) && (msgToSend))
        {
            TRIPRoutes tRoute((*itr)->getAddrFamily(), (*itr)->getAppProtocol(),                               TRIPData((*itr)->getPrefix()));
            if (withdrawn)
            {
                *wRoute << tRoute;
            }
            else
            {
                *rRoute << tRoute;
            }
        }
        else
        {
            prevTranId = (*itr)->getTransactionId();
            if (msgToSend)
            {
                uMsg = new TRIPUpdateMsg(wRoute, rRoute,
                                                route->getAttributes(), true);
                retList->push_back(uMsg);
            }

            wRoute = new TRIPWithdrawnRoute();
            rRoute = new TRIPReachableRoute();
            route = (*itr);
            msgToSend = true;
            TRIPRoutes tRoute((*itr)->getAddrFamily(), (*itr)->getAppProtocol(),                               TRIPData((*itr)->getPrefix()));
            if (withdrawn)
            {
                *wRoute << tRoute;
            }
            else
            {
                *rRoute << tRoute;
            }
        }
    }
    if (route.getPtr())
    {
        uMsg = new TRIPUpdateMsg(wRoute, rRoute, route->getAttributes(), true);
        retList->push_back(uMsg);
    }
    cpLog(LOG_DEBUG, "getMsgList - Number of advertise  msgs is (%d) ",
                          retList->size());
    return retList;
}

