/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const DecisionAgent_cxx_Version = 
"$Id: DecisionAgent.cxx,v 1.35 2001/03/20 22:46:28 kevinmc Exp $";

#include <iostream.h>
#include <strstream.h>
#include "DecisionAgent.hxx"
#include "TRIPUpdateMsg.hxx"
#include "TRIBAgent.hxx"
#include "TRIPTrieCache.hxx"
#include "LSUtils.hxx"
#include "TRIPWorkSet.hxx"
#include "cpLog.h"

DecisionAgent* DecisionAgent::_instance = 0;

DecisionAgent&
DecisionAgent::instance()
{
    if (!_instance)
    {
        _instance = new DecisionAgent();
    }
    return (*_instance);
}

DecisionAgent::DecisionAgent():_floodingFunc(0), extTribWorkSet(0)
{
}

void
DecisionAgent::destroy(void)
{
    delete DecisionAgent::_instance;
    DecisionAgent::_instance = 0;
}


void
DecisionAgent::process(const TRIPItadKey& key, Sptr <TRIPWorkSet> workSet, 
                                                    TRIPProcessReason reason)
{
    cpLog(LOG_DEBUG, "DecisionAgent::process for (%s)",
                                        key.getDescription().c_str());
    if (!(workSet.getPtr()))
    {
        assert(0);
    }
    TRIBAgent& tribAgent = TRIBAgent::instance();
    Sptr<TRIPTrieCache> adjTribIn;
    tribAgent.writeLock();
    try
    {
        adjTribIn = tribAgent.getTribIn(key);
    }
    catch (VMissingDataException e)
    {
        cpLog(LOG_INFO, "DecisionAgent -no adj-TribIn exists for (%s), DecisionAgent complete",                                                
                                                key.getDescription().c_str());
        tribAgent.unlock();
        return;
    }
    assert(adjTribIn->isUpdated());
    adjTribIn->setUpdated(false);
    tribAgent.unlock();
    const  RouteSetMap& workSetMap = workSet->getRouteSet();
    extTribWorkSet = new TRIPWorkSet();
    for (RouteSetMap::const_iterator itr = workSetMap.begin();
                                itr != workSetMap.end(); itr++)
    {
        if (adjTribIn->isInternalRouteCache() == false)
        {
            updateExtTrib(key, (*itr).second);
            updateExtToLocTrib( (*itr).first );
        }
        else
        {
            updateIntToLocTrib(key, (*itr).second);
        }
    }
    
    if (_floodingFunc != 0)
    {
        if (extTribWorkSet->getRouteSet().size() > 0)
        {
            Sptr <TRIPTrieCache> extTrib = tribAgent.getExtTrib();
            extTrib->writeLock();
            TRIPItadKey key(0, TRIPId());
            cpLog(LOG_DEBUG, "Calling floodingCallback");
            _floodingFunc(extTribWorkSet, TRIP_CB_EXT_TRIB_UPDATE, key);
            extTrib->unlock();
        }
    }
    if (reason == TRIP_CONNECTION_LOST)
    {
        tribAgent.writeLock();
        tribAgent.destroyAdjTribIn(key);
        tribAgent.unlock();
    }
    cpLog(LOG_DEBUG, "DecisionAgent::process complete for (%s)", 
    
                                            key.getDescription().c_str());
}

void
DecisionAgent::updateExtTrib(const TRIPItadKey& key, 
                                                const TRIPRouteSet& routeSet)
{
    cpLog(LOG_DEBUG, "DecisionAgent::updateExtTrib");
    TRIBAgent& tribAgent = TRIBAgent::instance();
    tribAgent.readLock();
    Sptr<TRIPTrieList > remainingTrie;
    remainingTrie = tribAgent.getSelectTrieListOmit(key);
    Sptr <TRIPTrieCache> extTrib = tribAgent.getExtTrib();
    RouteChangeBitMask srcMask = RouteChangeBitMask(T_UPDATE_BIT);
    RouteChangeBitMask destMask = RouteChangeBitMask(T_EXTTRIB_BIT
                                                                | T_FLOOD_BIT);
    extTrib->writeLock();
    processRouteSet(routeSet, remainingTrie, *extTrib, srcMask, destMask, true);
    extTrib->unlock();
    tribAgent.unlock();
}

void
DecisionAgent::updateIntToLocTrib(const TRIPItadKey& key, 
                                            const TRIPRouteSet& routeSet)
{
    cpLog(LOG_DEBUG, "DecisionAgent::updateIntToLocTrib");
    TRIBAgent& tribAgent = TRIBAgent::instance();
    tribAgent.writeLock();
    Sptr<TRIPTrieList > remainingTrie = tribAgent.getSelectTrieListOmit(key);
    RouteChangeBitMask srcMask = RouteChangeBitMask(T_UPDATE_BIT);
    RouteChangeBitMask destMask = RouteChangeBitMask(T_AGGR_BIT);
    Sptr <TRIPTrieCache> locTrib = tribAgent.getLocTrib();
    locTrib->writeLock();
    processRouteSet(routeSet, remainingTrie, *locTrib, srcMask, destMask, false);
    locTrib->unlock();
    tribAgent.unlock();
}

void
DecisionAgent::updateExtToLocTrib(const TRIPProtoKey& protoKey)
{
    cpLog(LOG_DEBUG, "DecisionAgent::updateExtToLocTrib");
    TRIBAgent& tribAgent = TRIBAgent::instance();
    tribAgent.readLock();
    Sptr<TRIPTrieCache> updatedTrieCache = tribAgent.getExtTrib();
    updatedTrieCache->writeLock();
    Sptr <TRIPRouteSet> srcRouteSet = updatedTrieCache->getRouteSet(protoKey);
    Sptr<TRIPTrieList > remainingTrie = tribAgent.getAllInternalTribIns();
    RouteChangeBitMask srcMask = RouteChangeBitMask(T_EXTTRIB_BIT);
    RouteChangeBitMask destMask = RouteChangeBitMask(T_AGGR_BIT);
    Sptr <TRIPTrieCache> locTrib = tribAgent.getLocTrib();
    locTrib->writeLock();
    processRouteSet(*srcRouteSet, remainingTrie, *locTrib, srcMask, destMask, false);
    locTrib->unlock();
    updatedTrieCache->unlock();
    tribAgent.unlock();
}
    
    
void 
DecisionAgent::processRouteSet(const TRIPRouteSet& updatedSet, 
                                Sptr<TRIPTrieList > remainingTrie,
                                TRIPTrieCache& destinationTrib,
                                const RouteChangeBitMask& srcMask,
                                const RouteChangeBitMask& destMask,
                                bool flooding)
{
    cpLog(LOG_DEBUG, "DecisionAgent::processRouteSet");
    for(TRIPRouteSet::iterator itr = updatedSet.begin();
                                                itr != updatedSet.end(); itr++)
    {
        Sptr<TRIPRouteElement> route = (*itr);
        if(route->isChanged(srcMask)) // Updated entry, should be processed
        {
            route->unsetChanged(srcMask); //reset Updated field

            // when matching routes exist the route
            // with highest preference is selected
            // if no match exists the current route is selected if it is
            // not being withdrawn
            Sptr<TRIPRouteElement> preferredRoute = 0;

            // get the preferred route for each updated destination
            TRIPRouteSet retSet = findMatching(route, remainingTrie);
            if (retSet.size() > 0)
            {
                preferredRoute = selectRoute(route, retSet);
            }
            // if no match and route is added it is the preferred route
            else if(route->isWithdrawn() == false)
            {
                preferredRoute = route;
            }

            // if withdrawn route has no route to replace it then it
            // needs to be withdrawn from the destinationTrib
            if (preferredRoute == 0)
            {
                withdrawRoutes(route, destinationTrib, destMask, flooding);
            }
            // otherwise replace the current routes with newly
            // selected preferred routes
            else
            {
                addRoutes(preferredRoute, destinationTrib, destMask, flooding);
            }
        }
    }
}

void 
DecisionAgent::withdrawRoutes(Sptr<TRIPRouteElement> route, 
                                    TRIPTrieCache& destinationTrib,
                                    const RouteChangeBitMask& destMask,
                                    bool flooding)
{
    cpLog(LOG_DEBUG, "DecisionAgent::withdrawRoutes");
    TRIPRouteSet retSet;
    if (destinationTrib.matchDestination(route, retSet)) 
    {
        if (retSet.size() > 1)
        {
            cpLog(LOG_ALERT, "Too many matching routes in destinationTrib");
            assert(0);
        }
        for(TRIPRouteSet::const_iterator itr = retSet.begin();itr != retSet.end(); itr++)
        {
            (*itr)->setChanged(destMask); 
            destinationTrib.withdraw((*itr));
            if (flooding)
            {
                extTribWorkSet->insert((*itr));
            }         
        }
    }
}

void
DecisionAgent::addRoutes(Sptr<TRIPRouteElement> route, 
                                    TRIPTrieCache& destinationTrib, 
                                    const RouteChangeBitMask& destMask,
                                    bool flooding)
{
    cpLog(LOG_DEBUG, "DecisionAgent::addRoutes");
    TRIPRouteSet retSet;
    if (destinationTrib.matchDestination(route, retSet))
    {
        if (retSet.size() > 1)
        {
            cpLog(LOG_ALERT, "Too many matching routes in destinationTrib");
            assert(0);
        }
        //if the selected route is already in the destTrib, no need to insert
        TRIPRouteSet::const_iterator itr = retSet.begin();
        if ((route.getPtr()) == ((*itr).getPtr()))
        {
            cpLog(LOG_DEBUG, "DecisionAgent::addRoutes - route already exists");
            return;
        }
        destinationTrib.remove((*itr));
    }
    route->setChanged(destMask);
    destinationTrib.insert(route);
    if (flooding)
    {
        extTribWorkSet->insert(route);
    }
}
    

Sptr<TRIPRouteElement>
DecisionAgent::selectRoute(Sptr<TRIPRouteElement> route, TRIPRouteSet& routeSet)
{
    cpLog(LOG_DEBUG, "DecisionAgent::selectRoute");
    Sptr<TRIPRouteElement> selectedRoute = 0;
    // if rte is being withdrawn then do not include it in list
    // from which route for this destination is to be selected
    if (route->isWithdrawn() == false) 
    {
        routeSet.insert(route);
    }
    // if no routes in selection list then no route selected
    if (routeSet.size() == 0)
    {
        return selectedRoute;
    }

    cpLog(LOG_ALERT, "DecisionAgent::selectRoute from (%d) routes", 
    routeSet.size());
    u_int32_t highestPref = 0; // Invalid value lower than all valid values

    for(TRIPRouteSet::iterator itr = routeSet.begin();
                                    itr != routeSet.end(); itr++)
    {
        Sptr<TRIPRouteElement> rte = (*itr);
        // Get route's Local Preference value
        Sptr<TRIPLocalPref> localPref;
        localPref.dynamicCast(rte->getAttribute(TRIP_LOCAL_PREF));
        assert (localPref.getPtr() != 0);
        u_int32_t localPrefValue = localPref->getPrefNumber();

        if (localPrefValue > highestPref)
        {
            highestPref = localPrefValue;
            selectedRoute = rte;
        }
        else if (localPrefValue == highestPref)
        {
            selectedRoute = selectRouteBreakingTie(selectedRoute, rte);
        }
    }
    return selectedRoute;
}

Sptr<TRIPRouteElement>
DecisionAgent::selectRouteBreakingTie(Sptr<TRIPRouteElement> left, 
                                    Sptr<TRIPRouteElement> right)
{
    Sptr<TRIPRouteElement> selectedRoute = left;
    if (left->getSource().isLocal())
    {
        if (right->getSource().isLocal()) // route originated internally
        {
            if(right->getSource().getTripId().getId() < 
                                    left->getSource().getTripId().getId())
            {
                selectedRoute = right;
            }
        }
    }
    else
    {
        if (right->getSource().isLocal()) 
        {
            selectedRoute = right;
        }
        else // both routes external
        {
            u_int16_t rightItad = right->getSource().getItad();
            u_int16_t leftItad = left->getSource().getItad();
            if (rightItad < leftItad)
            {
                selectedRoute = right;
            }
            else if(rightItad == leftItad)
            {
                if (multiExitDiscUsedForDecision())
                {
                    Sptr<TRIPMultiExitDisc> rightMultiExit = 0;
                    Sptr<TRIPMultiExitDisc> leftMultiExit = 0;
                    rightMultiExit.dynamicCast(right->
                                    getAttribute(TRIP_MULTI_EXIT_DISC));
                    if (rightMultiExit.getPtr())
                    {
                        leftMultiExit.dynamicCast(left->
                                        getAttribute(TRIP_MULTI_EXIT_DISC));
                        if (leftMultiExit.getPtr())
                        {
                            if (rightMultiExit->getPrefNumber() >
                            leftMultiExit->getPrefNumber())
                            {
                                selectedRoute = right;
                            }
                        }
                        else
                        {
                            selectedRoute = right;
                        }
                    }
                }
            }
        }
    }
    return selectedRoute;
}

bool
DecisionAgent::multiExitDiscUsedForDecision()
{
    cpLog(LOG_DEBUG, "DecisionAgent::multiExitDiscUsedForDecision");
    // TODO need to check config for setting indicating
    // use of MultExitDisc at this LS??? Default to true..
    return true;
}

TRIPRouteSet
DecisionAgent::findMatching(Sptr<TRIPRouteElement> route, 
                                    Sptr<TRIPTrieList> list)
{
    TRIPRouteSet retSet;
    for(TRIPTrieList::iterator itr = list->begin(); itr != list->end(); itr++)
    {
        TRIPRouteSet matchSet;
        if ((*itr)->matchDestination(route, matchSet))
        {
            if(matchSet.size() > 0)
            {
                retSet.insert(*(matchSet.begin()));
            }
        }
    }
    return retSet;
}

DecisionAgent::~DecisionAgent()
{
}

