#ifndef DecisionAgent_hxx
#define DecisionAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const DecisionAgent_hxx_Version = 
"$Id: DecisionAgent.hxx,v 1.19 2001/03/07 21:41:55 kevinmc Exp $";

#include "global.h"
#include <string>

#include "vtypes.h"
#include "Sptr.hxx"
#include "TRIPItadKey.hxx"
#include "TRIPRouteElement.hxx"
#include "TRIPWorkSet.hxx"
#include "TRIPDef.hxx"

class TRIPTrieCache;

/** 
    Class DecisionAgent is responsible for the stage of decision process
    which takes place when the contents of the AdjTribIn are used to  
    provide input to the ExtTrib and the LocTrib.
*/

class DecisionAgent {

    public:
        ///
        static DecisionAgent& instance();

        ///
        string className() { return "DecisionAgent"; };

        ///
        void process(const TRIPItadKey& key, Sptr<TRIPWorkSet> workSet,
                                TRIPProcessReason reason=TRIP_UPDATE_MSG);

        ///
        void registerForFlooding(FloodCallbackFunc func) { _floodingFunc = func; };

        ///
        static void destroy(void);

    private:
        ///Destructor
        virtual ~DecisionAgent();

        ///
        void updateExtTrib(const TRIPItadKey& key, 
                                                const TRIPRouteSet& routeSet);

        ///
        void updateIntToLocTrib(const TRIPItadKey& key, 
                                                const TRIPRouteSet& routeSet);

        ///
        void updateExtToLocTrib(const TRIPProtoKey& protoKey);

        ///
        void processRouteSet(const TRIPRouteSet& updatedSet,
                                        Sptr<TRIPTrieList> remainingTrie,
                                        TRIPTrieCache& destinationTrib,
                                        const RouteChangeBitMask& srcMask,
                                        const RouteChangeBitMask& destMask,
                                        bool flooding);

        ///
        void withdrawRoutes(Sptr<TRIPRouteElement> route,
                                        TRIPTrieCache& destinationTrib,
                                        const RouteChangeBitMask& destMask,
                                        bool flooding);

        ///
        void addRoutes(Sptr<TRIPRouteElement> route,
                                        TRIPTrieCache& destinationTrib,
                                        const RouteChangeBitMask& destMask,
                                        bool flooding);

        ///
        bool multiExitDiscUsedForDecision();

        ///
        Sptr<TRIPRouteElement> selectRoute(Sptr<TRIPRouteElement> route,
                                        TRIPRouteSet& routeSet);

        ///select route with from group with same preference
        Sptr<TRIPRouteElement> selectRouteBreakingTie
                    (Sptr<TRIPRouteElement> left, Sptr<TRIPRouteElement> right);

        ///
        TRIPRouteSet findMatching(Sptr<TRIPRouteElement> route,
                                        Sptr<TRIPTrieList > list);

        ///
        DecisionAgent();

        ///
        static DecisionAgent* _instance;

        ///
        FloodCallbackFunc _floodingFunc;

        ///
        Sptr <TRIPWorkSet> extTribWorkSet;

        ///
        friend LSSessionManager;

};


#endif
