/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const DecisionProxyAgent_cxx_Version = 
"$Id: DecisionProxyAgent.cxx,v 1.18 2001/03/09 18:57:15 kevinmc Exp $";

#include <iostream.h>
#include <strstream.h>
#include "DecisionProxyAgent.hxx"
#include "IncomingPolicyAgent.hxx"
#include "DecisionAgent.hxx"
#include "TRIPUpdateMsg.hxx"
#include "TRIPItadKey.hxx"
#include "TRIPUpdateEvent.hxx"
#include "TRIPWorkSet.hxx"

#include "cpLog.h"

DecisionProxyAgent* DecisionProxyAgent::_instance = 0;

DecisionProxyAgent&
DecisionProxyAgent::instance()
{
    if (!_instance)
    {
        _instance = new DecisionProxyAgent();
    }
    return (*_instance);
}

DecisionProxyAgent::DecisionProxyAgent():_decision(false), _waiting(false),
                                                            _shutdown(false)
{
    _evtFifo = new Fifo < Sptr < TRIPUpdateEvent > > ;
    _thread = new VThread();
    _thread->spawn(DecisionProxyAgent::run, this);
}


DecisionProxyAgent::~DecisionProxyAgent()
{
    if ((_evtFifo.getPtr()) && _waiting)
    {
        _shutdown = true;
        _evtFifo->add(0);
    }
    if (_thread.getPtr())
    {
        _thread->join();
        _thread->exit();
    }
}

void
DecisionProxyAgent::destroy(void)
{
    delete DecisionProxyAgent::_instance;
    DecisionProxyAgent::_instance = 0;
}

void
DecisionProxyAgent::addToQueue(Sptr< TRIPUpdateEvent > evt)
{
    cpLog(LOG_DEBUG, "DecisionProxyAgent::addToQueue");
    _evtFifo->add(evt);
}

void*
DecisionProxyAgent::run(void* args)
{
    cpLog(LOG_DEBUG, "DecisionProxyAgent::run");
    DecisionProxyAgent* self = static_cast < DecisionProxyAgent* > (args);
    while (true)
    {
        self->_waiting = true;
        cpLog(LOG_DEBUG, "DecisionProxyAgent::getNext Event from Fifo");
        Sptr < TRIPUpdateEvent > nextEvent = self->_evtFifo->getNext();
        self->_waiting = false;
        if (self->_shutdown) break;
        assert( nextEvent.getPtr() != 0 );
        self->process(nextEvent);
    }
    return (NULL);
}
 
void 
DecisionProxyAgent::process(Sptr < TRIPUpdateEvent > evt)
{
    cpLog(LOG_DEBUG, "DecisionProxyAgent::process");
    // retrieve ref to update msg and to itad key
    Sptr <TRIPUpdateMsg> update = evt->getUpdateMsg();
    const TRIPItadKey& key = evt->getItadKey();
    TRIPProcessReason reason = evt->getReason();
    setDecisionOn(false);
    Sptr <TRIPWorkSet> workSet = new TRIPWorkSet();
    // IncomingPolicyAgent should process the Update msg
    IncomingPolicyAgent& inAgt = IncomingPolicyAgent::instance();
    inAgt.process(update, key, workSet, reason);

    // If Decision is set to run then DecisionAgent 
    // must process the changed AdjTribIn represented by TRIPItadKey
    if (decisionSetToRun() == true)
    {
        DecisionAgent& decAgt = DecisionAgent::instance();
        decAgt.process(key, workSet);
    }
    
}    

void
DecisionProxyAgent::setDecisionOn(bool flg)
{
    _decision = flg;
}

bool 
DecisionProxyAgent::decisionSetToRun() const
{
    return _decision;
}
