#ifndef DecisionProxyAgent_hxx
#define DecisionProxyAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const DecisionProxyAgent_hxx_Version = 
"$Id: DecisionProxyAgent.hxx,v 1.8 2001/02/27 19:06:56 kevinmc Exp $";

#include "global.h"
#include <string>

#include "vtypes.h"
#include "Sptr.hxx"
#include "Fifo.h"
#include "TRIPUpdateEvent.hxx"

/** 
    Class DecisionProxyAgent is responsible for the decision process
    which takes place when an Update msg is received from an External
    or Internal Peer, or when new routes are injected locally. Given
    these routes the DecisionProxyAgent has the responsibility for
    determining which routes to use locally, which routes to 
    flood internally to other LS in the same ITAD, and which routes
    to disseminate to external Peers. DecisionProxyAgent runs in its
    own thread and makes calls to IncomingPolicyAgent and DecisionAgent
    during processing.
*/

class DecisionProxyAgent {

    public:
        ///Get the single instance of this class
        static DecisionProxyAgent& instance();

        ///Return a string containing this class name
        string className() { return "DecisionProxyAgent"; }

        ///Destructor
        virtual ~DecisionProxyAgent();

        ///Destroy the single instance of the class
        static void destroy(void);

        ///Add an Event to the Fifo monitored by the instance of this class
        void addToQueue(Sptr< TRIPUpdateEvent > evt);

        ///Set a flag indicating that the DecisionAgent process is to be run
        void setDecisionOn(bool flg);

    protected:
        ///Check if the flag for DecisionAgent process is set
        bool decisionSetToRun() const;

        ///Continuously monitor the Fifo by spawning a thread on this function
        static void* run(void* args);

        ///Constructor
        DecisionProxyAgent();

        ///Extract the Update Msg and TRIPItadKey and process appropriately
        void process(Sptr < TRIPUpdateEvent > evt);

        ///Single instance of this class
        static DecisionProxyAgent* _instance;

        ///Fifo for this class
        Sptr < Fifo < Sptr < TRIPUpdateEvent > > > _evtFifo;

        ///Thread for object of the class
        Sptr<VThread> _thread;

        ///Flag indicating whether DecisionAgent process is to be run
        bool _decision;

        ///Flag indicating if waiting for Fifo event
        bool _waiting;

        ///Flag indicating that object is being destructed
        bool _shutdown;
};


#endif
