#ifndef IncomingPolicyAgent_hxx
#define IncomingPolicyAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const IncomingPolicyAgent_hxx_Version = 
"$Id: IncomingPolicyAgent.hxx,v 1.18 2001/03/07 21:46:11 kevinmc Exp $";

#include "global.h"
#include <string>

#include "Sptr.hxx"
#include "TRIPItadKey.hxx"
#include "TRIPRouteElement.hxx"
#include "TRIPWorkSet.hxx"
#include "TRIPDef.hxx"


/** 
    Class IncomingPolicyAgent is responsible for the portion of decision 
    process which takes place when the contents of an Update msg are 
    inserted into AdjTribIn.LS has Adj-Trib-In associated with each 
    internal and external LS, it is peering with.
    Based on which peer the msg came from, and processing
    routes are either withdrawn, replaced or added into 
    corresponding AdjTribIn.
*/

class IncomingPolicyAgent {

    public:
        ///Get single instance of this object
        static IncomingPolicyAgent& instance();

        ///Return string containing class name
        string className() { return "IncomingPolicyAgent"; }

        ///Process withdrawn and reachable routes in Update Msg
        void process(Sptr <TRIPUpdateMsg> update, const TRIPItadKey& key,
                                    Sptr <TRIPWorkSet> workSet, 
                                    TRIPProcessReason reason=TRIP_UPDATE_MSG);

        /// Register for flooding when adjTribin is updated
        void registerForFlooding(FloodCallbackFunc func) { _floodingFunc = func; };

    private:
        ///Destructor
        virtual ~IncomingPolicyAgent() { };

        ///Initialize data required by the object of this class
        void init();

        ///Check that internal route is valid based on Sequence Number
        bool internalRouteIsValid(const Sptr<TRIPRouteElement> route, 
        Sptr<TRIPTrieCache> tribIn, bool& matchedDest);

        ///Insert withdrawn route in appropriate adjTribIn
        bool handleWithdrawnRoute(const Sptr<TRIPRouteElement> newRoute,
                                Sptr <TRIPTrieCache> tribIn,
                                bool isInternal, Sptr <TRIPWorkSet> workSet);

        ///Insert reachable route in appropriate adjTribIn
        bool handleReachableRoute(const Sptr<TRIPRouteElement> newRoute,
                                Sptr <TRIPTrieCache> tribIn,
                                bool isInternal, Sptr <TRIPWorkSet> workSet);

        ///Constructor
        IncomingPolicyAgent();

        ///Calculate the preference of a route based on policy data
        u_int32_t calculatePref(const TRIPRouteElement& route);

        ///Lost connection to external Peer - withdraw all routes
        void handleLostConnection(const TRIPItadKey& key,
                                                    Sptr <TRIPWorkSet> workSet);

        ///Single instance of IncomingPolicyAgent
        static IncomingPolicyAgent* _instance;

        ///Call Back function called to process the Flooding function
        FloodCallbackFunc _floodingFunc;

        ///
        friend LSSessionManager;

};


#endif
