/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSApiServer_cxx_Version = 
"$Id: LSApiServer.cxx,v 1.6 2001/02/14 20:21:08 sprajpat Exp $";


#include <unistd.h>
#include <iostream>
#include <stdlib.h> 
#include <strstream.h>
#include "LSLocator.hxx"
#include "LSApiServer.hxx"
#include "LSSendOnlySession.hxx"
#include "LSReceiveOnlySession.hxx"
#include "LSSendReceiveSession.hxx"
#include "TcpServerSocket.hxx"
#include "TRIPOpenMsg.hxx"
#include "TRIPTransceiver.hxx"
#include "cpLog.h"

LSApiServer* LSApiServer::_instance = 0;

extern "C" {
    void LSApiServer_destroy()
    {
        LSApiServer::destroy();
    }
};

LSApiServer&
LSApiServer::instance(int port)
{
    if (!_instance)
    {
        _instance = new LSApiServer(port);
        cpLog(LOG_INFO, "Started LSApiServer on port (%d)", port);
    }
    return (*_instance);
}

LSApiServer&
LSApiServer::instance()
{
    assert(_instance != 0);
    return (*_instance);
}

void 
LSApiServer::destroy(void)
{
    delete LSApiServer::_instance;
    LSApiServer::_instance = 0;
}

LSApiServer::LSApiServer(int port)
{
    _port = port;
    _shutdown = false;
    if (atexit(LSApiServer_destroy) < 0)
    {
        cpLog(LOG_ALERT, "Failed to register with atexit()");
    };

    _serverSocket = new TcpServerSocket(port);
    _thread = new VThread();
    _thread->spawn(LSApiServer::run, this);
}

LSApiServer::~LSApiServer()
{
    _shutdown = true;
    _serverSocket->close();
    if(_thread.getPtr() != 0)
    {
        _thread->join();
        _thread->exit();
    }
}

void*
LSApiServer::run(void* args)
{
    LSApiServer* self = static_cast<LSApiServer*>(args);
    self->processMsg();
    return (0);
}

void
LSApiServer::processMsg()
{
    //Currently the logic assumes that A Proxy Server will be pulling
    //information from the Location Server.However if it changes
    //into a PUSH model, where Location server will be pushing data
    //into Proxy, the logic need to be redesigned to fit the requirement
    for (; ; )
    {
        if(_serverSocket->getServerConn().isReadReady(1))
        {
            Connection conn;
            _serverSocket->accept(conn);
            cpLog(LOG_DEBUG, "Accepted connection (%s)", 
                              conn.getDescription().c_str());
            _connectionList.push_back(conn);
            continue;
        }
        if(_shutdown ) break;
        try
        {
            for(ConnectionList::iterator itr = _connectionList.begin();
                                itr != _connectionList.end(); itr++)
            {
                if((*itr).isLive() && (*itr).isReadReady(1))
                {
                    processRequest((*itr));
                }
            }

            for(ConnectionList::iterator itr = _connectionList.begin();
                                itr != _connectionList.end(); itr++)
            {
                if(!(*itr).isLive())
                {
                    _connectionList.erase(itr);
                    break;
                }
            }
        }
        catch(VNetworkException& e)
        {
            cpLog(LOG_ALERT, "Network error, reason:%s",
                             e.getDescription().c_str());
        }
        catch(VException& e)
        {
            cpLog(LOG_ALERT, "Caught exception, reason:%s",
                             e.getDescription().c_str());
        }
    }
}

void
LSApiServer::processRequest(Connection& conn) throw (VNetworkException&)
{
    cpLog(LOG_DEBUG, "Processing request on conn (%s)",
                                 conn.getDescription().c_str());
    char buf[1];
    int bRead;
    TRIPUtil::readData(conn.getConnId(), buf, 1, bRead);
    if(bRead == 0)
    {
         cpLog(LOG_INFO, "Closing conenction (%s)", conn.getDescription().c_str());
         conn.close();
         return;
    }
}

