/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSTopology_cxx_Version = 
"$Id: LSTopology.cxx,v 1.12 2001/03/08 01:28:09 kevinmc Exp $";


#include <stdio.h>
#include <strstream.h>
#include "LSTopology.hxx"
#include <algorithm>
#include "cpLog.h"

LSTopology::LSTopology(const TRIPId& tripId):itsTripId(tripId)
{
}

LSTopology::~LSTopology()
{
}

bool 
LSTopology::isConnected(const TRIPId& src, const TRIPId& dest)
{
    bool connected = false;
    readLock();
    if(_mapOfLSTopology.count(src))
    {
        Sptr <LSConnectionList> connList = _mapOfLSTopology[src];
        if (connList->find(dest) != connList->end())
        {
            connected = true;
        }
    }
    unlock();
    return connected;
}

Sptr <LSConnectionList>
LSTopology::getConnectionList(const TRIPId& src)
{
    Sptr <LSConnectionList> copyList = 0;
    readLock();
    if(_mapOfLSTopology.count(src))
    {
        copyList = new LSConnectionList();
        Sptr <LSConnectionList> connList = _mapOfLSTopology[src];
        for (LSConnectionList::const_iterator itr = connList->begin();
                                            itr != connList->end(); itr++)
        {
            copyList->insert((*itr));
        }
    }
    unlock();
    return copyList;
}

bool
LSTopology::isReachable(const TRIPId& dest)
{
    bool reached = false;
    LSConnectionList destList;
    destList.insert(dest);
    readLock();
    if (canBeReached(destList))
    {
        reached = true;
    }
    return reached;
}

void
LSTopology::addConnection(const TRIPId& src, const TRIPId& dest)
{
    writeLock();
    addBothConnections(src, dest);
    unlock();
}

void 
LSTopology::addBothConnections(const TRIPId& src, const TRIPId& dest)
{
    // handle src - dest connection
    if(_mapOfLSTopology.count(src))
    {
        Sptr <LSConnectionList> connList = _mapOfLSTopology[src];
        if (connList->find(dest) == connList->end())
        {
            connList->insert(dest);
        }
    }
    else
    {
        Sptr <LSConnectionList> connList = new LSConnectionList();
        connList->insert(dest);
        _mapOfLSTopology.insert(pair<TRIPId, Sptr<LSConnectionList > > 
                                                            (src, connList));
    }
    //handle dest - src connection
    if(_mapOfLSTopology.count(dest))
    {
        Sptr <LSConnectionList> connList = _mapOfLSTopology[dest];
        if (connList->find(src) == connList->end())
        {
            connList->insert(src);
        }
    }
    else
    {
        Sptr <LSConnectionList> connList = new LSConnectionList();
        connList->insert(src);
        _mapOfLSTopology.insert(pair<TRIPId, Sptr<LSConnectionList > >
                                                            (dest, connList));
    }
}

void 
LSTopology::removeSetOfConnections(const TRIPId& src)
{
    if(_mapOfLSTopology.count(src))
    {
        Sptr <LSConnectionList> connList = _mapOfLSTopology[src];
        for (LSConnectionList::const_iterator itr = connList->begin();
                            itr != connList->end(); itr++)
        {
            removeBothConnections((*itr), src);
        }
        _mapOfLSTopology.erase(src);
        _mapOfSequenceNumbers.erase(src);
    }
}

void 
LSTopology::removeConnection(const TRIPId& dest, LSConnectionList& removeList)
{
    writeLock();
    for (TopologyMap::const_iterator itr = _mapOfLSTopology.begin();
                                    itr != _mapOfLSTopology.end(); itr++)
    {
        if (((*itr).first) != itsTripId)
        {
            removeList.insert(((*itr).first));
        }
    }
    removeBothConnections(itsTripId, dest);
    if (!(canBeReached(removeList)))
    {
        for (LSConnectionList::const_iterator itr = removeList.begin();
                                            itr != removeList.end(); itr++)
        {
            removeSetOfConnections((*itr));
        }
    }
    unlock();
}
     
void 
LSTopology::removeBothConnections(const TRIPId& src, const TRIPId& dest)
{
    if(_mapOfLSTopology.count(src))
    {
        Sptr <LSConnectionList> connList = _mapOfLSTopology[src];
        if (connList->find(dest) != connList->end())
        {
            connList->erase(dest);
            if (connList->size() == 0)
            {
                _mapOfLSTopology.erase(src);
                _mapOfSequenceNumbers.erase(src);
            }
        }
    }
    if(_mapOfLSTopology.count(dest))
    {
        Sptr <LSConnectionList> connList = _mapOfLSTopology[dest];
        if (connList->find(src) != connList->end())
        {
            connList->erase(src);
            if (connList->size() == 0)
            {
                _mapOfLSTopology.erase(dest);
                _mapOfSequenceNumbers.erase(dest);
            }
        }
    }
}

bool
LSTopology::updateTopology(Sptr <TRIPItadTopology> newTopology,
                     LSConnectionList& addList, LSConnectionList& removeList)
{
    if (newTopology.getPtr())
    {
        const TRIPId& src = newTopology->getTripId();
        u_int32_t sequenceNum = newTopology->getSeqNumber();
        if (src == itsTripId)
        {
            cpLog(LOG_DEBUG, "UpdateTopology - topology originated at this LS");
            return false;
        }
        
        writeLock();
        if (!(_mapOfSequenceNumbers.count(src)))
        {
            _mapOfSequenceNumbers.insert(pair<TRIPId, u_int32_t> (src, sequenceNum));
        }
        else
        {
            if (sequenceNum <= _mapOfSequenceNumbers[src])
            {
                cpLog(LOG_DEBUG, "UpdateTopology - Invalid sequence number");
                unlock();
                return false;
            }
            else
            {
                _mapOfSequenceNumbers.erase(src);
                _mapOfSequenceNumbers.insert(pair<TRIPId, u_int32_t> (src, sequenceNum));
            }
        }
        const TripIdList& destList = newTopology->getTripIdList();
        Sptr <LSConnectionList> newList = new LSConnectionList();
        for (TripIdList::const_iterator itr = destList.begin();
                                        itr != destList.end(); itr++)
        {
            newList->insert((*itr));
        }
        if (!(_mapOfLSTopology.count(src)))
        {
            addList = *newList;
        }
        else
        {
            Sptr <LSConnectionList> oldList = _mapOfLSTopology[src];
            set_difference(newList->begin(), newList->end(),
                            oldList->begin(), oldList->end(),
                            inserter(addList, addList.begin()));
            set_difference(oldList->begin(), oldList->end(),
                            newList->begin(), newList->end(),
                            inserter(removeList, removeList.begin()));
        }
        for (LSConnectionList::const_iterator itr = removeList.begin();
                                        itr != removeList.end(); itr++)
        {
            removeBothConnections(src, (*itr));
        }
        for (LSConnectionList::const_iterator itr = addList.begin();
                                        itr != addList.end(); itr++)
        {
            addBothConnections(src, (*itr));
        }
        if (removeList.size() > 0)
        {
            //insert all dests in removelist and check for reachability
            //to each one - when not reachable then it is removed
            for (TopologyMap::const_iterator itr = _mapOfLSTopology.begin();
                                    itr != _mapOfLSTopology.end(); itr++)
            {
                if (((*itr).first) != itsTripId)
                {
                    removeList.insert(((*itr).first));
                }
            }
            if (!(canBeReached(removeList)))
            {
                for (LSConnectionList::const_iterator itr = removeList.begin();
                                            itr != removeList.end(); itr++)
                {
                    removeSetOfConnections((*itr));
                }
            }
        }
        unlock();
        ostrstream oStr;
        oStr << *this << endl;
        oStr << ends;
        cpLog(LOG_DEBUG, "Topology Updated:\n%s", oStr.str());
        oStr.freeze(false);
        return true;
    }
    return false;
} 

bool
LSTopology::canBeReached(LSConnectionList& notReachable)
{
    if (_mapOfLSTopology.size() == 0)
    {
        return false;
    }
    Sptr <LSConnectionList> searchList = new LSConnectionList();
    Sptr <LSConnectionList> searchedList = new LSConnectionList();
    //begin the search at the tripid for this LS
    searchList->insert(itsTripId);
      
    bool searching = true;
    while((searchList->size() > 0) && (searching))
    {
        Sptr <LSConnectionList> tempList = new LSConnectionList();
        //search each connectionList associated with the tripids in the 
        //searchList - delete tripids from the notReachable list as found.
        //tripids from the searchlist are placed in the searchedList
        //so that the connectionList for this tripid is not searched again
        //tripids remaining in notReachable List at the end are not reachable
        for (LSConnectionList::const_iterator itr = searchList->begin();
                                        itr != searchList->end(); itr++)
        {
            searchedList->insert((*itr));
            if(_mapOfLSTopology.count((*itr)) == 0) continue; 
            Sptr <LSConnectionList> connList = _mapOfLSTopology[(*itr)];
            for (LSConnectionList::const_iterator dItr = notReachable.begin();
                                        dItr != notReachable.end(); dItr++)
            {
                if (connList->find(*dItr) != connList->end())
                {
                    notReachable.erase(*dItr);
                }
                if (notReachable.size() == 0)
                {
                    searching = false;
                    break;
                }
            }
            for (LSConnectionList::const_iterator nItr = connList->begin();
                                            nItr != connList->end(); nItr++)
            {
                //if not already searched for then need to search
                if(searchedList->find(*nItr) == searchedList->end())
                {
                    //if currently searching then do not search again
                    if (searchList->find(*nItr) == searchList->end())
                    {
                        tempList->insert((*nItr));
                    }
                }
            }
        }
        searchList = tempList;
    }
    return (notReachable.size() == 0);
}

void 
LSTopology::readLock() const
{
    cpLog(LOG_DEBUG, "Locking LS Topology Map read-only");
    _lock.ReadLock();
}


void 
LSTopology::writeLock()
{
    cpLog(LOG_DEBUG, "Locking LS Topology Map read-write");
    _lock.WriteLock();
}


void 
LSTopology::unlock() const
{
    cpLog(LOG_DEBUG, "Unlocking LS Topology Map");
    _lock.Unlock();
}

void
LSTopology::dump(ostream& os) const
{
    readLock();
    os.width(25);
    os << "LS Topology" << endl;
    for (TopologyMap::const_iterator itr = _mapOfLSTopology.begin();
                                    itr != _mapOfLSTopology.end(); itr++)
    {
        os.width(25);
        os << (*itr).first << ": ";
        for (LSConnectionList::iterator connItr = ((*itr).second)->begin();
                                connItr != ((*itr).second)->end(); connItr++)
        {
            os << "->" << (*connItr).getId();
        }
        os << endl;
    }
    unlock();
}
