#ifndef LSTopology_hxx
#define LSTopology_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSTopology_hxx_Version = 
"$Id: LSTopology.hxx,v 1.5 2001/02/15 00:54:06 kevinmc Exp $";

#include "global.h"
#include <map>
#include <set>
#include <string>
#include <stdio.h>

#include "Sptr.hxx"
#include "TRIPId.hxx"
#include "TRIPItadTopology.hxx"
#include "TRIPDef.hxx"
#include "VRWLock.hxx"

/** 
    Class implements the connection graph for the Itad Topology
*/
class LSTopology {
    public:

        ///Map of LS(TRIPid) with associated connection lists
        typedef map <TRIPId, Sptr<LSConnectionList > > TopologyMap;

        ///Map of LS with associated sequence Numbers
        typedef map <TRIPId, u_int32_t> SequenceMap;

        ///Constructor
        LSTopology(const TRIPId& tripId);

        ///Destructor
        virtual ~LSTopology();

        ///Indication of connection existing between the src and dest LS
        bool isConnected(const TRIPId& src, const TRIPId& dest);

        ///Indicate if the dest is reachable from this LS
        bool isReachable(const TRIPId& dest);

        ///Add a connection between src and dest LS
        void addConnection(const TRIPId& src, const TRIPId& dest);

        ///
        void removeConnection(const TRIPId& dest, 
                                                LSConnectionList& removeList);

        ///Process TRIPItadTopology data to update LS Topology representation
        bool updateTopology(Sptr <TRIPItadTopology> newTopology,
                     LSConnectionList& addList, LSConnectionList& removeList);

        ///
        Sptr <LSConnectionList> getConnectionList(const TRIPId& src);

        ///
        string className() { return "LSTopology"; }

        ///
        void readLock() const;

        ///
        void writeLock();

        ///
        void unlock() const;

        ///
        friend ostream& operator<<(ostream& os, const LSTopology& topology);

        ///
        void dump(ostream& os) const;

    private:
        ///Check if given LS is reachable from this LS
        bool canBeReached(LSConnectionList& notReachable);

        ///Remove a set of connections associated with a TRIPid
        void removeSetOfConnections(const TRIPId& src);

        ///Remove the connection between src and dest LS
        void removeBothConnections(const TRIPId& src, const TRIPId& dest);

        ///Add bothway connections from src - dest and dest - src
        void addBothConnections(const TRIPId& src, const TRIPId& dest);

        ///Map of TRIPId and Connection List
        TopologyMap _mapOfLSTopology;

        SequenceMap _mapOfSequenceNumbers;

        ///
        mutable VRWLock     _lock;

        ///
        TRIPId      itsTripId;

};

inline ostream& operator<<(ostream& os, const LSTopology& topology)
{
    topology.dump(os);
    return os;
}

#endif
