
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const LSUtils_cxx_Version = 
"$Id: LSUtils.cxx,v 1.18 2001/02/07 23:26:35 sprajpat Exp $";
#include <assert.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include "LSUtils.hxx"
#include "TRIPUpdateMsg.hxx"
#include "TRIPWithdrawnRoute.hxx"
#include "LSSessionManager.hxx"
#include "cpLog.h"

Sptr <TRIPRouteSet >
LSUtils::translateRoutes(const TRIPItadKey& source, const TRIPUpdateMsg& uMsg)
{  
    const u_int32_t trId = uMsg.getTransactionId();
    Sptr <TRIPRouteSet >  retSet = 
                  new TRIPRouteSet();
    bool itrip = false;
    TRIPItadKey orig;
    u_int32_t seqNum = 0;
    //First take care of withdrawn routes
    const Sptr<TRIPWithdrawnRoute>& wRoute = uMsg.getWithdrawnRoute();
    if(wRoute->isALinkStateAttr())
    {
        itrip = true;
        orig = TRIPItadKey(LSSessionManager::instance().getItad(),wRoute->getTripId()); 
        seqNum = wRoute->getSeqNumber();
    }

    const TRIPRouteContainer::RouteList& rList = wRoute->getRouteList();
    Sptr<TRIPRouteElement> rElement;
    for(TRIPRouteContainer::RouteList::const_iterator itr = rList.begin();
           itr != rList.end(); itr++)
    {
       rElement = new TRIPRouteElement(source, (*itr), "", 
                                       uMsg.getAttributes(), itrip); 
       rElement->setWithdrawn(true);
       rElement->setOriginator(orig);
       rElement->setSeqNumber(seqNum);
       rElement->setTransactionId(trId);
       retSet->insert(rElement); 
    }
    //Take care of reachable routes
    const Sptr<TRIPReachableRoute>& rRoute = uMsg.getReachableRoute();
    Sptr<TRIPRouteAttr> rAttr = uMsg.getAttribute(TRIP_NEXT_HOP_SERVER);
    Sptr<TRIPNextHopServer> nHopAttr;
    nHopAttr.dynamicCast(rAttr);
    assert(nHopAttr != 0);
    Data nHop = nHopAttr->getHopAddress().str();
    const TRIPRouteContainer::RouteList& rList2 = rRoute->getRouteList();
    for(TRIPRouteContainer::RouteList::const_iterator itr = rList2.begin();
           itr != rList2.end(); itr++)
    {
       rElement = new TRIPRouteElement(source, (*itr),nHop, 
                                       uMsg.getAttributes(), 
                                       itrip); 
       retSet->insert(rElement); 
       rElement->setOriginator(orig);
       rElement->setSeqNumber(seqNum);
       rElement->setTransactionId(trId);
    }
    return retSet;
}

