#ifndef OutgoingPolicyAgent_hxx
#define OutgoingPolicyAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OutgoingPolicyAgent_hxx_Version = 
"$Id: OutgoingPolicyAgent.hxx,v 1.10 2001/02/14 01:13:08 kevinmc Exp $";

#include "global.h"
#include <string>

#include "vtypes.h"
#include "Sptr.hxx"
#include "OutgoingPolicyTimerAgent.hxx"
#include "TRIPItadKey.hxx"
#include "TRIPRouteElement.hxx"
#include "TRIPDef.hxx"

class TRIPMsg;
class TRIPUpdateMsg;

/** 
    Class OutgoingPolicyAgent is responsible for creating
    routes in Adj-TRib_outs that eventually get advertised to the external Peer
    sessions.  For each external peer session, it loads the set of policies at the 
    time of initialization.As the Loc trib is processed for each peer session, the
    policy is applied and routes are put in the Adj-trib-out.
*/
class OutgoingPolicyAgent {

    public:
        ///
        static OutgoingPolicyAgent& instance();

        ///
        string className() { return "OutgoingPolicyAgent"; }

        ///
        void addNewPeerKey(const TRIPItadKey& key);

    protected:
        ///
        static void destroy(void);

        ///
        virtual ~OutgoingPolicyAgent();

        ///
        void init();

        ///
        static void* run(void* args);

        ///
        OutgoingPolicyAgent();

        ///
        void policyEnforcer();

        ///
        void processPolicyRules(Sptr <TRIPRouteElement> route, 
                            Sptr<TRIPTrieCache> dest, const TRIPItadKey& itad);

        ///
        void processNewPeers();

        ///
        void populateAdjTribOut(const TRIPItadKey& key);

        ///
        static OutgoingPolicyAgent* _instance;

        ///
        Sptr<VThread> _thread;

        ///
        Sptr <VThreadEvent> runPolicy;

        ///
        Sptr <OutgoingPolicyTimerAgent> timerAgt;

        ///
        vector<TRIPItadKey> newPeerKeyList;

        ///
        bool _shutdown;

        ///
        friend LSSessionManager;
};


#endif
