#ifndef TRIBAgent_hxx
#define TRIBAgent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TRIBAgent_hxx_Version = 
"$Id: TRIBAgent.hxx,v 1.17 2001/03/22 01:04:28 kevinmc Exp $";

#include "global.h"
#include <map>
#include <string>

#include "vtypes.h"
#include "Sptr.hxx"
#include "TRIPItadKey.hxx"
#include "TRIPTrieCache.hxx"
#include "VMissingDataException.hxx"
#include "TRIPDef.hxx"

/** 
    Singelton class, implements the interface to all the route databases like
    ADJ-Trib-In, ADJ-Trib-out, Loc-Trib and Ext-Trib in a TRIP instance.
*/
class TRIBAgent {
    public:
        ///
        static TRIBAgent& instance();

        ///
        string className() { return "TRIBAgent"; }

        /**Returns a ADJ-TRIB-In for a given Peer, throws exception
         * if Route list is not found.
         */
        Sptr<TRIPTrieCache> getTribIn(const TRIPItadKey& key) 
                         throw (VMissingDataException&);

        ///
        void createAdjTribIn(const TRIPItadKey& key, TCacheType cType);

        ///
        void destroyAdjTribIn(const TRIPItadKey& key);

        ///
        void destroyAdjTribOut(const TRIPItadKey& key);

        /**Returns a ADJ-TRIB-Out for a given Peer, throws exception
         * if Route list is not found.
         */
        Sptr<TRIPTrieCache> getTribOut(const TRIPItadKey& key) 
                         throw (VMissingDataException&);

        ///
        void createAdjTribOut(const TRIPItadKey& key, TCacheType cType);

        ///Returns a LOC-TRIB 
        Sptr <TRIPTrieCache> getLocTrib() ;

        ///Returns a Ext-TRIB 
        Sptr <TRIPTrieCache> getExtTrib() ;

        ///Returns a union of all the ADJ-TRIB-In
        Sptr<TRIPTrieList> getAllInternalTribIns() const;

        ///
        Sptr<TRIPTrieList> getSelectTrieListOmit(const TRIPItadKey& key);

        ///
        void prettyDump(ostream& os) const;

        ///
        void readLock() const;

        ///
        void writeLock() const;

        ///
        void unlock() const;

    protected:
        ///
        TRIBAgent();

        ///Destructor
        virtual ~TRIBAgent();

        /**
           Frees the memory associated with singelton instance.
           gets register to atexit() function at the time of creation.
        */
        static void destroy(void);

        ///
        static TRIBAgent* _instance;

        ///Map of ADJ-TRIB-Ins
        TRIBTrieCacheIn _mapOfTribIn;

        ///Map of ADJ-TRIB-Outs
        TRIBTrieCacheIn _mapOfTribOut;

        ///
        Sptr <TRIPTrieCache>   _locTrib;

        ///
        Sptr <TRIPTrieCache>   _extTrib;

        ///
        mutable VRWLock     _lock;

        ///
        friend LSSessionManager;
};

#endif
