#ifndef TRIPDef_HXX
#define TRIPDef_HXX
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIBDef_hxx_Version = 
"$Id: TRIPDef.hxx,v 1.19 2001/03/07 22:04:09 kevinmc Exp $";

#include <map>
#include <set>
#include <list>
#include <vector>
#include <math.h>

#include "Sptr.hxx"
#include "TRIPItadKey.hxx"
#include "TRIPRouteHdr.hxx"
#include "TRIPProtoKey.hxx"
#include "TRIPId.hxx"

class TRIPTrieCache;
class TRIPRouteElement;
class TRIPRouteAttr;
class LSPeerSession;
class TRIPUpdateMsg;
class TRIPWorkSet;

///
typedef map <TRIPItadKey, Sptr<TRIPTrieCache> > TRIBTrieCacheIn;

///
typedef set<Sptr<TRIPRouteElement> > TRIPRouteSet;

///
typedef map<TRIPProtoKey, TRIPRouteSet > RouteSetMap;

///
typedef vector<TRIPAppProto> ProtoCapabilityList;

///
typedef vector<TRIPAddrFamily> AddrFamilyList;

///
typedef set< Sptr<TRIPRouteAttr> > AttributeSet;

///
typedef vector<Sptr<LSPeerSession> >  LSPeerList;

///
typedef vector<Sptr<TRIPTrieCache> >  TRIPTrieList;

///
typedef vector<Sptr<TRIPUpdateMsg> > UpdateMsgList;

///
typedef list<TRIPId> TripIdList;

///
typedef set<TRIPId > LSConnectionList;


///
typedef set<TRIPItadKey>  TRIPItadTopologySet;


typedef enum
{
    TRIP_CB_INTERNAL_UPDATE = 0,
    TRIP_CB_EXTERNAL_UPDATE,
    TRIP_CB_LOC_TRIB_UPDATE,
    TRIP_CB_EXT_TRIB_UPDATE,
    TRIP_CB_TRIB_OUT_UPDATE
}TRIPCallbackReason;

typedef enum
{
    TRIP_UPDATE_MSG = 0,
    TRIP_CONNECTION_LOST
}TRIPProcessReason;

typedef enum
{   
    T_NON_AGGREGATED=0x01,
    T_AGGREGATED =0x02,
    T_AGGREGATED_RES =0x04 
}TAggrStatus;

typedef enum
{
  TC_INTERNAL=1,
  TC_LOCAL,
  TC_EXTERNAL
} TCacheType;




///
typedef void(*FloodCallbackFunc)(Sptr <TRIPWorkSet> workSet, 
                      TRIPCallbackReason reason, const TRIPItadKey& changedSrc);

//One could have used Max limits defined in limits.h, however the definition of
//Max in limits.h will depend on the platfrom app is running and so following
//definition would give a Max value for 32 bit int.
///
static const u_int32_t MAX_TRIPID_VALUE = static_cast<u_int32_t>(pow(2, 32) - 1);

///
static const u_int16_t MAX_ITAD_VALUE = static_cast<u_int16_t>(pow(2, 16) - 1);

///
static const int DEFAULT_KEEPALIVE_TIME=80000;     //milliseconds
///
static const int DEFAULT_HOLD_TIME=240000;     //milliseconds
///
static const int DEFAULT_CONN_RETRY_TIME=3000;     //milliseconds
///
static const int DEFAULT_RESTART_TIME=60000;    //milliseconds

///Minimum time where Restart Timer and Connect Retry Timer stop growing
static const int DEFAULT_MAX_TIME=240000;

static const int OUTGOING_POLICY_TIMER=30000; //milliseconds

#endif
