/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <stdio.h>

#include "TRIPItadKey.hxx"
#include "TRIPUtil.hxx"
#include "LSSessionManager.hxx"

TRIPItadKey::TRIPItadKey(u_int16_t itad, const TRIPId& tripId)
            : _itad(itad), 
              _tripId(tripId) , 
              _internal(false), 
              _local(false), 
              _initLocal(false),
              _initInternal(false)
{
}


TRIPItadKey::TRIPItadKey(const TRIPItadKey& other) 
{
    copyObj(other);
}

TRIPItadKey& 
TRIPItadKey::operator=(const TRIPItadKey& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return (*this);
}

void
TRIPItadKey::copyObj(const TRIPItadKey& other)
{
    _itad = other._itad;
    _tripId = other._tripId;
    _internal = other._internal;
    _local = other._local;
    _initLocal = other._initLocal;
    _initInternal = other._initInternal;
    _strRep = other._strRep;
}

const string&
TRIPItadKey::getDescription() const
{
    if(_strRep.size()) return _strRep;
    char buf[56];
    sprintf(buf, "(%d, %u)", _itad, _tripId.getId());
    _strRep = buf;
    return _strRep;
}

///
void 
TRIPItadKey::dump(ostream& os) const
{
    os << getDescription().c_str();
}

bool 
TRIPItadKey::isLocal() const
{
    if(!_initLocal)
    {
        if(*this == LSSessionManager::instance().getTripItadKey())
        {
            _local = true;
        }
        _initLocal = true;
    }
    return _local;
}

bool 
TRIPItadKey::isInternal() const
{
    if(!_initInternal)
    {
        if(_itad == LSSessionManager::instance().getItad())
        {
            _internal = true;
        }
        _initInternal = true;
    }
    return _internal;
}

