#ifndef TRIPItadKey_hxx
#define TRIPItadKey_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPItadKey_hxx_Version = 
"$Id: TRIPItadKey.hxx,v 1.16 2001/02/10 02:35:19 kevinmc Exp $";

#include <netinet/in.h>
#include <string>

#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPId.hxx"

/** 
  A TRIPIdadKey is a composite key of ITAD and TRIPId.
  Routes are first sorted based on the ITAD number and for the same 
  ITAD number sorting is done based on the TRIPId.
 */

class TRIPItadKey {
    public:
        ///
        TRIPItadKey(u_int16_t itad, const TRIPId& tripId);

        ///
        TRIPItadKey(const TRIPItadKey& other) ;

        ///
        TRIPItadKey& operator=(const TRIPItadKey& other);

        ///
        bool operator==(const TRIPItadKey& other) const
        {
            return((_itad == other._itad) && (_tripId == other._tripId));
        }

        ///
        bool operator<(const TRIPItadKey& other) const
        {
            if(_itad < other._itad) return true;
            if(_itad == other._itad)
            {
                if(_tripId < other._tripId) return true;
            }
            return false;
        }

        ///
        bool operator!=(const TRIPItadKey& other) const
        {
            return (!operator==(other));
        }

        /**@name Accessors */
        //@{

        ///
        u_int16_t getItad() const { return _itad; };

        ///
        const TRIPId& getTripId() const { return _tripId; };

        ///
        const string& getDescription() const;

        ///
        bool isLocal() const;

        ///
        bool isInternal() const;

        //@}

        ///
        friend ostream& operator<<(ostream& os, const TRIPItadKey&);

        ///
        void dump(ostream& os) const;

    private:
        ///
        TRIPItadKey(): _itad(0), _internal(false), _local(false) { };

        ///
        void copyObj(const TRIPItadKey& other);

        ///
        u_int16_t _itad;

        ///
        TRIPId    _tripId;

        ///
        mutable string    _strRep;

        ///
        mutable bool  _internal;

        ///
        mutable bool  _local;

        ///
        mutable bool _initLocal;

        ///
        mutable bool _initInternal;

        ///
        friend class TRIPRouteElement;
        ///
        friend class LSPeerSession;
        ///
        friend class TRIPTrieCache;
        ///
        friend class LSSessionManager;
        ///
        friend class LSUtils;
};

inline ostream& operator<<(ostream& os, const TRIPItadKey& obj) 
{
    obj.dump(os);
    return os;
}

#endif
