#ifndef TRIPProtoKey_hxx
#define TRIPProtoKey_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPProtoKey_hxx_Version = 
"$Id: TRIPProtoKey.hxx,v 1.1 2001/02/03 00:04:13 sprajpat Exp $";

#include <string>
#include "vtypes.h"
#include "TRIPRouteHdr.hxx"

/** 
  A TRIPProtoKey is a composite key of Application protocol and
  and prefix type ( POTS or Routing number).
 */
class TRIPProtoKey {
    public:
        ///
        TRIPProtoKey(TRIPAppProto proto, TRIPAddrFamily aFamily)
            : _proto(proto), _aFamily(aFamily) { };

        ///
        TRIPProtoKey(const TRIPProtoKey& other) ;

        ///
        TRIPProtoKey& operator=(const TRIPProtoKey& other);

        ///
        inline bool operator==(const TRIPProtoKey& other) const
        {
            return((_proto == other._proto) &&
                   (_aFamily == other._aFamily));
        }

        ///
        inline bool operator<(const TRIPProtoKey& other) const
        {
            if(_proto < other._proto) return true;
            if(_proto == other._proto)
            {
                if(_aFamily < other._aFamily) return true;
            }
            return false;
        }

        /**@name Accessors */
        //@{

        ///
        TRIPAppProto getProto() const { return _proto; };

        ///
        TRIPAddrFamily getAddrFamily() const { return _aFamily; };

        ///
        const string& getDescription() const;

        //@}

        ///
        friend ostream& operator<<(ostream& os, const TRIPProtoKey&);

        ///
        void dump(ostream& os) const;

    private:
        ///
        TRIPProtoKey();

        ///
        TRIPAppProto _proto;

        ///
        TRIPAddrFamily    _aFamily;

        ///
        mutable string    _strRep;

};

inline ostream& operator<<(ostream& os, const TRIPProtoKey& obj) 
{
    obj.dump(os);
    return os;
}

#endif
