#ifndef TRIPTrieElement_hxx
#define TRIPTrieElement_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPTrieElement_hxx_Version = 
"$Id: TRIPTrieElement.hxx,v 1.5 2001/02/09 03:26:00 sprajpat Exp $";

#include <assert.h>                                                 
#include <stdio.h> 

#include "vtypes.h"
#include "Sptr.hxx"
#include "TRIPDef.hxx" 
#include "TRIPTrieUtil.hxx"
#include "cpLog.h"         


/** Implements template base class for Trie node which stores data type<T>.
*/
template <class T>
class TRIPTrieElement 
{
   public:
      ///
      virtual ~TRIPTrieElement() { };

      /**@name Accessor */
      //@{

      ///
      virtual bool isANode() const { return false; };
      ///
      virtual bool isALeaf() const { return false; };

      ///
      virtual bool canBeAggregated() const { return false; };

      ///
      virtual bool isAggregated() const { return false; };

      ///
      char getCharId() const { return _charId; };

      ///
      int getId() const { return _id; };

      ///
      int getDepth() const { return _depth; };

      ///
      virtual Sptr<T> getValue() const { return _val; }


      //@}

      /**@name  Mutator*/
      //@{

      ///
      void setValue(Sptr<T> val) { _val = val; }

      /**Set the order of the suffix-tree should only be called
       * on the root node.
       */
      virtual void setOrder(int order);

      //@}

      ///
      bool operator<(const TRIPTrieElement& other)
      {
          return(_id < other._id);
      }

      ///
      bool operator==(const TRIPTrieElement& other)
      {
          return(_id ==  other._id);
      }

      ///
#if defined(__linux__)
      template<class T>
      friend  ostream& operator<<(ostream& os, const TRIPTrieElement<T>& obj);
#endif

      ///
      virtual void dump(ostream& os) const 
      { 
          if(getParent() == 0) 
          {
              return;
          }
          os.width(25);
          os << "**RouteElement**" << endl;
          os.width(25);
          os << "Char:" << getCharId() << endl;
          os.width(25);
          os << "Depth:" << getDepth() << endl;
          if(getValue() != 0)
          {
              os.width(25);
              getValue()->prettyDump(os);
          }
      };

      ///
      TRIPTrieElement* getParent() const { return _parent; }

   protected:
      ///
      TRIPTrieElement(char c, TRIPTrieElement* parent);

      ///
      TRIPTrieElement(int  val, TRIPTrieElement* parent);

      ///
      int  _id;
      ///
      char _charId;

      ///
      TRIPTrieElement* _parent;

      ///
      int _depth;

      ///
      int _order;

      ///
      bool _orderSet;

      ///
      Sptr<T> _val;
};


template <class T> 
inline ostream& operator<<(ostream& os, const TRIPTrieElement<T>& obj)
{
    obj.dump(os);
    return os;
}


#include "TRIPTrieElement.cxx"

#endif
