
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TRIPTrieNode_cxx_Version = 
"$Id: TRIPTrieNode.cxx,v 1.7 2001/02/14 20:02:28 sprajpat Exp $";

template <class T>
void 
TRIPTrieNode<T>::add(Sptr<TRIPTrieElement> elem)
{
    assert(elem->getId() < _order);
    elem->setOrder(_order);
    if(elem->isANode())
    {
        _numOfNodesCached = false;
        cpLog(LOG_DEBUG, "Inserting node (%d) into (%d) ", elem->getId(), _id);
        _nodeList[elem->getId()] = elem;
        cpLog(LOG_DEBUG, "Child count (%d) for node (%d) ", getNodeCount(), _id);
    }
}

template <class T>
void 
TRIPTrieNode<T>::remove(const TRIPTrieElement* elem)
{
    if(elem->isANode())
    {
        if(_nodeList[elem->getId()].getPtr() != 0) _nodeList[elem->getId()] = 0;
    }
}


template <class T>
void 
TRIPTrieNode<T>::remove(const string& pattern)
{
    const TRIPTrieNode* node = getDataInternal(pattern);
    int len = pattern.length();
    if(node->getDepth() == len)
    {
        TRIPTrieNode* parent = dynamic_cast<TRIPTrieNode*>(node->getParent());
        parent->remove(node);
    }
}

///
template <class T>
void 
TRIPTrieNode<T>::add(Sptr<T> elem)
{
    const string& prefix = elem->getPrefix();
    int sz = prefix.length();
    cpLog(LOG_DEBUG, "Inserting prefix (%s)", prefix.c_str());
    TRIPTrieNode* firstNode = this;
    for(int i = 0 ; i < sz; i++)
    {
        int k = TRIPTrieUtil::charToInt(prefix[i]); 
        Sptr<TRIPTrieElement> eElem = firstNode->_nodeList[k]; 
        if(eElem.getPtr() != 0)
        {
            firstNode = dynamic_cast<TRIPTrieNode*>(eElem.getPtr());
        }
        else
        {
            Sptr<TRIPTrieElement> nNode = new TRIPTrieNode(k, firstNode);
            firstNode->add(nNode);
            firstNode = dynamic_cast<TRIPTrieNode*>(nNode.getPtr());
        }
    }
    firstNode->setValue(elem);
}

template <class T>
void 
TRIPTrieNode<T>::dump(ostream& os) const
{
    TRIPTrieElement<T>::dump(os);
    if(getNodeCount())
    {
        os.width(25);
        os << "\t**Child node list**" << endl;
        for(int i = 0; i < _order; i++)
        {
            if(_nodeList[i].getPtr() != 0)
            {
                (_nodeList[i])->dump(os); 
            }
        }
    }
}

template <class T>
int 
TRIPTrieNode<T>::getNodeCount() const
{
    if(!_numOfNodesCached)
    {
        _numOfNodes = 0;
        for(int i = 0; i < _order; i++)
        {   
            if(_nodeList[i].getPtr() != 0) _numOfNodes++;
        }
        _numOfNodesCached = true;
    }
    return _numOfNodes;
}

template <class T>
void 
TRIPTrieNode<T>::setOrder(int order)
{  
    if(_orderSet) return;
    TRIPTrieElement::setOrder(order);
    _nodeList  = new Sptr<TRIPTrieElement>[_order];
    for(int i = 0; i < _order; i++)
    {   
        _nodeList[i] = 0;
    }
}

template <class T>
const TRIPTrieNode<T>* 
TRIPTrieNode<T>::getDataInternal(const string& prefix) const
{
    int sz = prefix.length();
    const TRIPTrieNode* firstNode  = this;
    for(int i = 0 ; i < sz; i++)
    {
        int k = TRIPTrieUtil::charToInt(prefix[i]); 
        Sptr<TRIPTrieElement> elem = firstNode->_nodeList[k]; 
        if(elem.getPtr() != 0)
        {
            firstNode = dynamic_cast<const TRIPTrieNode*>(elem.getPtr());
        }
        else
        {
            return firstNode;
        }
    }
    return firstNode;
}

template <class T>
Sptr<T> 
TRIPTrieNode<T>::getData(const string& prefix) const
{
    return(getDataInternal(prefix)->getValue());
}

template <class T>
Sptr<T> 
TRIPTrieNode<T>::getDataExact(const string& prefix) const
{
    const TRIPTrieNode* node = getDataInternal(prefix);

    //If the depth of the found node is not equal to the
    //length of the string , it is not a exact match
    int sz = prefix.length();
    if(node->getDepth() != sz) return 0;
    
    return(getDataInternal(prefix)->getValue());
}

template <class T>
void
TRIPTrieNode<T>::getDataSet(const TRIPTrieNode *root, set<Sptr<T> >& retSet, int flags) const
{
    for(int i = 0 ; i < root->_order; i++)
    {
        Sptr<TRIPTrieElement> elem = root->_nodeList[i];
        if(elem.getPtr() != 0)
        {
            Sptr<T> data = elem->getValue();
            if((data.getPtr() != 0) && (data->qualifies(flags)))
            {
                retSet.insert(data);
            }
            ///Recurse through its children
            TRIPTrieNode *nRoot = dynamic_cast<TRIPTrieNode *>(elem.getPtr());
            getDataSet(nRoot, retSet, flags);
        }
    }
}

template <class T>
Sptr<T> 
TRIPTrieNode<T>::getMoreSpecificTo(const string& prefix, bool& moreSpecificFlg) const
{
    const TRIPTrieNode* node = getDataInternal(prefix);
    int len = prefix.length();
    if((node->getDepth() == len) && (node->getNodeCount() == 0))
    {
        moreSpecificFlg = true;
        TRIPTrieNode* nNode = dynamic_cast<TRIPTrieNode*>(node->getParent());
        return(findValue(nNode));
    }
    moreSpecificFlg = false;
    return 0;
}

template <class T>
Sptr<T>
TRIPTrieNode<T>::findValue(const TRIPTrieNode* root) const
{
    Sptr<T> retVal;
    if(root->getValue().getPtr() != 0) return (root->getValue());
    if(root->getParent() == 0) return 0;
    TRIPTrieNode* nNode = dynamic_cast<TRIPTrieNode*>(root->getParent());
    return (findValue(nNode));
}

template <class T>
bool 
TRIPTrieNode<T>::isAggregatable(const string& prefix, string& parentPrefix,
                                Sptr<T>& route, 
                                set<Sptr<T> >& retSet)
{ 
    bool aggrFlg = false;
    const TRIPTrieNode* node = getDataInternal(prefix);
    int len = prefix.length();
    parentPrefix = prefix.substr(0, len-1);
    if(node->getDepth() == len)
    {
        //Exact match
        TRIPTrieNode* nNode = dynamic_cast<TRIPTrieNode*>(node->getParent());
        route = nNode->getValue();
        //Check to see if it has all the children
        if(nNode->getNodeCount() == _order)  aggrFlg = true;
        for(int i = 0 ; i < nNode->_order; i++)
        {
             Sptr<TRIPTrieElement> elem = nNode->_nodeList[i];
             if(elem.getPtr())
             {
                 Sptr<T> data = elem->getValue();
                 if (data.getPtr())
                 {
                    retSet.insert(data);
                 }
             }
        }
    }
    return aggrFlg; 
}
