#ifndef TRIPTrieNode_hxx
#define TRIPTrieNode_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPTrieNode_hxx_Version = 
"$Id: TRIPTrieNode.hxx,v 1.6 2001/02/14 20:02:30 sprajpat Exp $";

#include "TRIPTrieElement.hxx"

/** Template class derived from TRIPTrieElement implements a Trie node 
    which holds the data of type <T>.
*/
template <class T>
class TRIPTrieNode : public TRIPTrieElement<T>
{
   public:
      ///Creates the root node of a Suffix Tree
      TRIPTrieNode()
          : TRIPTrieElement('0', 0),
            _aggregated(false),
            _nodeList(0),
            _numOfNodes(0),
            _numOfLeaves(0),
            _numOfNodesCached(false),
            _numOfLeavesCached(false)
      { };
      ///
      TRIPTrieNode(char c, TRIPTrieElement* parent)
          : TRIPTrieElement(c, parent), 
            _aggregated(false),
            _nodeList(0),
            _numOfNodes(0),
            _numOfLeaves(0),
            _numOfNodesCached(false),
            _numOfLeavesCached(false)
      { };

      ///
      TRIPTrieNode(int  i, TRIPTrieElement* parent)
          : TRIPTrieElement(i, parent), 
            _aggregated(false),
            _nodeList(0),
            _numOfNodes(0),
            _numOfLeaves(0),
            _numOfNodesCached(false),
            _numOfLeavesCached(false)
      { };

      ///
      ~TRIPTrieNode()
      {
          clear();
          delete []_nodeList;
      }

      ///
      void clear()
      {
          for(int i = 0; i < _order; i++)
          {
              _nodeList[i] = 0;
          }
      }


      /**@name Accessor */
      //@{

      ///
      bool isANode() const { return true; };

      ///
      bool canBeAggregated() const 
      {
          //TODO
          return false;
      }


      ///
      bool isAggregated() const { return _aggregated; };

      ///
      void dump(ostream& os) const;

      ///
      int getNodeCount() const;


      /**Returns the Data element which best matches the prefix
       * for example if a tree with node 1408, if a prefix is searched for
       * "14081", it will return value from the node 1408.
       */
      Sptr<T> getData(const string& prefix) const;

      /**Returns the Data element which  matches the prefix
       * for example if a tree with node 1408, if a prefix is searched for
       * "14081", it will return 0
       */
      Sptr<T> getDataExact(const string& prefix) const;

      ///Gets the Data element which the given prefix is more specific
      Sptr<T> getMoreSpecificTo(const string& prefix, bool& moreSpecificFlg) const;

      ///
      Sptr<T> findValue(const TRIPTrieNode* root) const;


      ///Gets the data elements in the tree that qualifies the flags.
      void getDataSet(const TRIPTrieNode *root, set<Sptr<T> >& retSet, int flags) const;

      bool isAggregatable(const string& prefix, string& parentPrefix,
                                Sptr<T>& route,
                                set<Sptr<T> >& retSet);


      //@}

      /**@name  Mutator*/
      //@{
   
      ///
      void setAggregated(bool val) { _aggregated = val; }

      ///
      void add(Sptr<TRIPTrieElement> elem);


      ///
      void remove(const string& pattern);

      ///
      void add(Sptr<T> elem);

      /**Set the order of the suffix-tree should only be called
       * on the root node.
       */
      void setOrder(int order);


      //@}

    private:
      ///
      const TRIPTrieNode* getDataInternal(const string& prefix) const;

      ///
      void remove(const TRIPTrieElement* elem);

      ///
      bool _aggregated;
      ///
      Sptr<TRIPTrieElement>*  _nodeList;

      ///
      mutable int _numOfNodes;
      ///
      mutable int _numOfLeaves;

      ///
      mutable bool _numOfNodesCached;
      ///
      mutable bool _numOfLeavesCached;
};

#include "TRIPTrieNode.cxx"

#endif
