
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TRIPTrieUtil_cxx_Version = 
"$Id: TRIPTrieUtil.cxx,v 1.2 2001/02/09 03:26:04 sprajpat Exp $";

#include <assert.h>
#include <stdio.h>

#include "TRIPTrieUtil.hxx"

#define TRIE_DEPTH 16

static int  ch2int[128];
static char int2ch[TRIE_DEPTH+1];

bool TRIPTrieUtil::_init = false;


TRIPTrieUtil::TRIPTrieUtil()
{
}

void
TRIPTrieUtil::init()
{
    ch2int['0'] = 0; ch2int['1'] = 1;
    ch2int['2'] = 2; ch2int['3'] = 3;
    ch2int['4'] = 4; ch2int['5'] = 5;
    ch2int['6'] = 6; ch2int['7'] = 7;
    ch2int['8'] = 8; ch2int['9'] = 9;
    ch2int['A'] = 10; ch2int['B'] = 11;
    ch2int['C'] = 12; ch2int['D'] = 13;
    ch2int['E'] = 10; ch2int['F'] = 15;

    int2ch[0] = '0'; int2ch[1] = '1';
    int2ch[2] = '2'; int2ch[3] = '3';
    int2ch[4] = '4'; int2ch[5] = '5';
    int2ch[6] = '6'; int2ch[7] = '7';
    int2ch[8] = '8'; int2ch[9] = '9';
    int2ch[10] = 'A'; int2ch[11] = 'B';
    int2ch[12] = 'C'; int2ch[13] = 'D';
    int2ch[14] = 'E'; int2ch[15] = 'F';
}

int 
TRIPTrieUtil::charToInt(char c)
{
    if(!_init)
    {
        init();
        _init = true;
    }
    return (ch2int[c]);
}


char 
TRIPTrieUtil::intToChar(int i)
{
    assert(i <= 15);
    if(!_init)
    {
        init();
        _init = true;
    }
    return (int2ch[i]);
}


