#ifndef TRIPAttrHeader_hxx
#define TRIPAttrHeader_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPAttrHeader_hxx_Version = 
"$Id: TRIPAttrHeader.hxx,v 1.13 2001/01/23 02:10:28 sprajpat Exp $";

#include <netinet/in.h>
#include <iostream.h>
#include <assert.h>

#include "global.h"
#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"
#include "TRIPId.hxx"
#include "TRIPCoreDef.hxx"

static const int ATTR_HDR_LENGTH=4;

//Possible attribute flags
typedef enum {
    TRIP_FLG_DEFAULT=0x00,
    TRIP_FLG_OPT=0x01,
    TRIP_FLG_TRANSITIVE=0x02,
    TRIP_FLG_DEPENDENT =0x04,
    TRIP_FLG_PARTIAL=0x08,
    TRIP_FLG_LINK_STATE=0x10,
    TRIP_FLG_ALL=(TRIP_FLG_OPT | TRIP_FLG_TRANSITIVE | TRIP_FLG_DEPENDENT | TRIP_FLG_PARTIAL | TRIP_FLG_LINK_STATE),
    //Delimits the number of flags.A new flag MUST be defined here
    TRIP_FLG_MAX = 0x20   
} TRIPAttrFlag;

//Attribute types
typedef enum {
    TRIP_WITHDRAWN_ROUTE=1,
    TRIP_REACHABLE_ROUTE,
    TRIP_NEXT_HOP_SERVER,
    TRIP_ADVERTISEMENT_PATH,
    TRIP_ROUTED_PATH,
    TRIP_ATOMIC_AGGREGATE,
    TRIP_LOCAL_PREF,
    TRIP_MULTI_EXIT_DISC,
    TRIP_COMMUNITY,
    TRIP_ITAD_TOPOLOGY,
    TRIP_AUTHENTICATION,
    TRIP_ATTR_TYPE_MAX
} TRIPAttrType;


/** Implements the base class for all the attribute messages which are part
    of UPDATE message.
    <pre>
    +---------------------------------------------------------------------+
    + Attr flags (1 byte) | AttrType code (1 byte) | Attr length (2 bytes)+
    +---------------------------------------------------------------------+
    </pre>
*/
class TRIPAttrHeader {
    public:
        //Structure to represent the Attribute header data (4 bytes long)
        typedef struct _TripHAttrTag {
            u_int8_t  _flag;
            u_int8_t  _type;
            u_int16_t _length;
        } TripAttrHeaderData;
        ///
        TRIPAttrHeader( u_int8_t aFlg, TRIPAttrType type);

        ///
        virtual ~TRIPAttrHeader() { };

        /**@name Accessors */
        //@{
        ///
        string           getAttrFlagStr() const;
        ///
        u_int8_t         getAttrFlag() const;
        ///
        TRIPAttrType getAttrType() const;
        ///
        string           getAttrTypeStr() const;

        ///The return length of data only 
        u_int16_t getLength() const { return _dataRep._length; };

        ///The return length includes the length of the header also
        u_int16_t getAttrLength() const { return _dataRep._length+getHdrLength(); };

        ///Returns the fixed length of header
        inline virtual u_int16_t getHdrLength()  const  ;

        ///
        virtual void dump(ostream& os) const { };

        ///
        virtual bool isALinkStateHeader() const { return false; };

        ///
        virtual u_int32_t getSeqNumber() const { return 0; };

        ///
        virtual const TRIPId& getTripId() const  { return _tripId; };

        //@}


        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&);

    protected:
        ///
        friend ostream& operator<<(ostream& os, const TRIPAttrHeader&);

        ///
        TRIPAttrHeader() { };

        ///
        TRIPAttrHeader(const char* buf) throw (TRIPBadDataException&);

        ///
        TRIPAttrHeader(const TRIPAttrHeader& other); 

        ///
        TRIPAttrHeader& operator=(const TRIPAttrHeader& other);

        ///
        bool operator==(const TRIPAttrHeader& other) const;

        ///
        void setLength(u_int16_t len) { _dataRep._length = len; };

        ///
        void copyObj(const TRIPAttrHeader& other);

        ///
        TripAttrHeaderData _dataRep;

        ///
        TRIPId       _tripId;

        ///
        friend class TRIPTransceiver;
        ///
        friend class TRIPMsgParser;
        ///
        friend class TRIPRouteAttr;
};

inline ostream& operator<<(ostream& os, const TRIPAttrHeader& obj)
{
    os << "**Attribute HEADER**" << endl;
    os.width(DUMP_LABEL_WIDTH);
    os << "Flag: " << obj.getAttrFlagStr() << endl;
    os.width(DUMP_LABEL_WIDTH);
    os << "Type: " << obj.getAttrTypeStr() << endl;
    os.width(DUMP_LABEL_WIDTH);
    os << "Length: " << static_cast<int>(obj._dataRep._length) << endl;
    obj.dump(os);
    return os;
}

#endif
