#ifndef TRIPErrorHandler_hxx
#define TRIPErrorHandler_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPErrorHandler_hxx_Version = 
"$Id: TRIPErrorHandler.hxx,v 1.4 2001/01/16 03:26:03 sprajpat Exp $";

#include "vtypes.h"
#include "TRIPBadDataException.hxx"

//Possible error codes
typedef enum {
    TRIP_ERR_MSG_HEADER=1,
    TRIP_ERR_OPEN,
    TRIP_ERR_UPDATE,
    TRIP_ERR_HOLD_TIMER,
    TRIP_ERR_FSM,
    TRIP_ERR_CEASE,
    //Define new error code here
    TRIP_ERR_MAX
} TRIPErrorCode;

/** Singelton class handling the errcode validations as well 
    maintaing the error messages.
*/
class TRIPErrorHandler {
    public:
        ///
        static TRIPErrorHandler& instance();

        ///
        string className() { return "TRIPErrorHandler"; }


        ///
        void validateCodes(u_int8_t eCode, u_int8_t eSubCode)
                           throw (TRIPBadDataException& );
        ///
        string getErrorStr(u_int8_t eCode, u_int8_t eSubCode)
                           throw (TRIPBadDataException&);

        ///
        string getSubcodeStr(u_int8_t eCode, u_int8_t eSubCode)
                           throw (TRIPBadDataException&);

        ///Destructor
        ~TRIPErrorHandler();
    private:

        ///
        TRIPErrorHandler();
     
        ///
        static TRIPErrorHandler* _instance;
};

#endif
