#ifndef TRIPItadPathSegment_hxx
#define TRIPItadPathSegment_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPItadPathSegment_hxx_Version = 
"$Id: TRIPItadPathSegment.hxx,v 1.11 2001/01/23 02:10:41 sprajpat Exp $";

#include <list>
#include <iostream.h>
#include <assert.h>

#include "global.h"
#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"
#include "TRIPCoreDef.hxx"


//Allowed segment types
typedef enum {
    TRIP_SEG_AP_SET = 1,
    TRIP_SEG_AP_SEQ,
    //Add a new segment type here
    TRIP_SEG_MAX
} TRIPPathSegType;

static const int SEGMENT_MIN_LENGTH=2;

/** Implements the ITAD path segments of a Routed/Advertisement path.
    It represents a triplet consist of type-length-value.The value is a 
    list of one or more ITADs.
    <pre>    
    +------------------------------------------------------------+
    + type (1 byte)  | length (1 byte) | Value (variable length) +
    +------------------------------------------------------------+
    </pre>
*/
class TRIPItadPathSegment {
    public:
        ///
        typedef list<u_int16_t> ITAD_LIST;

        ///
        TRIPItadPathSegment(TRIPPathSegType type);

        ///
        TRIPItadPathSegment(TRIPPathSegType type, u_int16_t itad);

        ///
        TRIPItadPathSegment(const TRIPItadPathSegment& other); 

        ///
        TRIPItadPathSegment& operator=(const TRIPItadPathSegment& other);

        ///
        virtual ~TRIPItadPathSegment() { };

        /**@name Accessors */
        //@{
        ///
        u_int16_t getLength() const { return(SEGMENT_MIN_LENGTH + (_numOfItads * 2)); }

        ///
        u_int8_t getType() const { return _type; };

        ///Returns true if given itad is part of the segment
        bool containsItad(u_int16_t itad) const;

	///Returns false if the Itad list contains any elements
	bool isEmpty() const;

        ///
        const ITAD_LIST& getItadList() const { return _itadList; }

        //@}

        /**@name Mutators */
        //@{
        ///
        void addItad(u_int16_t itad);
        //@}

        ///
        friend ostream& operator<<(ostream& os, const TRIPItadPathSegment&);

        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&);

        ///
        bool operator==(const TRIPItadPathSegment& other) const ;

    protected:
        ///
        TRIPItadPathSegment(const char* buf) throw (TRIPBadDataException&);

	///
	void setType(TRIPPathSegType type);

        ///
        void copyObj(const TRIPItadPathSegment& other);

        ///
        TRIPPathSegType  _type;
        ///
        u_int8_t         _numOfItads;
        ///
        ITAD_LIST        _itadList;
 
        ///
        friend class TRIPPathContainer;

};

inline ostream& operator<<(ostream& os, const TRIPItadPathSegment& obj)
{
    os.width(DUMP_LABEL_WIDTH);
    os << "**ITAD-PathSegment**" << endl;
    os.width(DUMP_LABEL_WIDTH);
    os << "Type: " << static_cast<int>(obj._type) << endl;
    os.width(DUMP_LABEL_WIDTH);
    os << "length: " << static_cast<int>(obj._numOfItads) << endl;

    int idx = 0;
    os.width(DUMP_LABEL_WIDTH);
    os << "Itad list:";
    for(TRIPItadPathSegment::ITAD_LIST::const_iterator itr = obj._itadList.begin(); itr != obj._itadList.end(); itr++)
    {
        if(idx) os << ",";
        if( idx  == 10)
        {
            os << endl;
            os.width(DUMP_LABEL_WIDTH);
            idx = 0;
        }
        os << static_cast<int>(*itr);
        idx++;
    }
    os << endl;
    return os;
}

#endif
