/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPMultiExitDisc_cxx_Version = 
"$Id: TRIPMultiExitDisc.cxx,v 1.6 2001/03/05 20:21:19 kevinmc Exp $";

#include <netinet/in.h>

#include "TRIPMultiExitDisc.hxx"
#include "TRIPUtil.hxx"
#include "TRIPErrorHandler.hxx"
#include "cpLog.h"

TRIPMultiExitDisc::TRIPMultiExitDisc(const TRIPMultiExitDisc& other) 
{
     copyObj(other);
}

TRIPMultiExitDisc& 
TRIPMultiExitDisc::operator=(const TRIPMultiExitDisc& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPMultiExitDisc::dump(ostream& os) const
{
    os.width(DUMP_LABEL_WIDTH);
    os << "**MultiExitDisc**" << endl;
    os.width(DUMP_LABEL_WIDTH);
    os << "Value:" << static_cast<int>(_prefNumber) << endl;
}

void 
TRIPMultiExitDisc::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    u_int32_t prefNum =  htonl(_prefNumber);
    TRIPUtil::writeData(id, &prefNum, MULTI_LNK_PREF_LENGTH);
}

TRIPMultiExitDisc::TRIPMultiExitDisc(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    assert(buf);

    if(_hdr->getAttrType() != TRIP_MULTI_EXIT_DISC)
    {
        throw TRIPBadDataException("Bad Multi-Exit-Disc attribute",
                                   __FILE__, __LINE__, TRIP_ERR_UPDATE);
    }

    int hdrLen  = _hdr->getHdrLength(); 
    memcpy(&_prefNumber, buf+hdrLen, MULTI_LNK_PREF_LENGTH);
    _prefNumber = ntohl(_prefNumber);
}

void 
TRIPMultiExitDisc::copyObj(const TRIPMultiExitDisc& other)
{
    TRIPRouteAttr::copyObj(other);
    _prefNumber = other._prefNumber;
}

bool
TRIPMultiExitDisc::isEqualAttribute(const TRIPRouteAttr& attr) const
{       
    assert(attr.getType() == TRIP_MULTI_EXIT_DISC);
    const TRIPMultiExitDisc* other = dynamic_cast<const TRIPMultiExitDisc*>(&attr);
    assert(other);
    if(!other) return false;
    return (TRIPRouteAttr::isEqualAttribute(attr) &&
            (_prefNumber == other->_prefNumber));
}

