#ifndef TRIPMultiExitDisc_hxx
#define TRIPMultiExitDisc_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPMultiExitDisc_hxx_Version = 
"$Id: TRIPMultiExitDisc.hxx,v 1.8 2001/02/16 21:18:22 sprajpat Exp $";

#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"
#include "TRIPRouteAttr.hxx"

static const int MULTI_LNK_PREF_LENGTH=4;

/** Derived from TRIPRouteAttr, implements a MultiExitDisc attribute.
    It contains a 4 bytes preference number.
    <pre>
    +-----------------------------------------------------------+
    +                 TRIPAttrHeader                            +
    +-----------------------------------------------------------+
    +               4 byte preference number                    +
    +-----------------------------------------------------------+
    </pre>
*/
class TRIPMultiExitDisc : public TRIPRouteAttr {
    public:
        ///Create a normal attribute
        TRIPMultiExitDisc(u_int32_t prefNumber, u_int8_t flg=TRIP_FLG_DEFAULT)
            : TRIPRouteAttr(flg, TRIP_MULTI_EXIT_DISC),
              _prefNumber(prefNumber)
        {
            setLength(MULTI_LNK_PREF_LENGTH);
        };

        ///
        ~TRIPMultiExitDisc() { };

        ///
        TRIPMultiExitDisc(const TRIPMultiExitDisc& other); 

        ///
        TRIPMultiExitDisc& operator=(const TRIPMultiExitDisc& other);

        //Accessors
        ///
        u_int32_t getPrefNumber() const { return _prefNumber; };

        ///
        void setPrefNumber(u_int32_t prefNum) { _prefNumber = prefNum; };

        ///
        void dump(ostream& os) const;

        ///Writes object to given network socket in network byte order
        void writeData(int id) const throw(VNetworkException&);
 
        ///
        bool isEqualAttribute(const TRIPRouteAttr& attr) const;

    private:
        ///
        TRIPMultiExitDisc(const char* buf) throw (TRIPBadDataException&);
            
        ///
        void copyObj(const TRIPMultiExitDisc& other);

        ///
        u_int32_t        _prefNumber;

        ///
        friend class TRIPUpdateMsg;
};

#endif
