/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPPathContainer_cxx_Version = 
"$Id: TRIPPathContainer.cxx,v 1.17 2001/03/06 02:50:18 kevinmc Exp $";

#include "TRIPPathContainer.hxx"
#include "TRIPItadPathSegment.hxx"
#include "TRIPErrorHandler.hxx"
#include "cpLog.h"


TRIPPathContainer::TRIPPathContainer(const TRIPPathContainer& other) 
{
     copyObj(other);
}

TRIPPathContainer& 
TRIPPathContainer::operator=(const TRIPPathContainer& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPPathContainer::addPathSegment(const TRIPItadPathSegment& data)
{
    _segmentList.push_back(data);
    _len += data.getLength();
    setLength(_len);
}

TRIPPathContainer&
TRIPPathContainer::operator<<(const TRIPItadPathSegment& data)
{
    addPathSegment(data);
    return *this;
}

void 
TRIPPathContainer::dump(ostream& os) const
{
    os.width(25);
    PathSegmentList::const_iterator itr;
    for(itr = _segmentList.begin(); 
              itr != _segmentList.end(); itr++)
    {
         const TRIPItadPathSegment& data = (*itr);
         os << data << endl;
    }
}

void 
TRIPPathContainer::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    for(PathSegmentList::const_iterator itr = _segmentList.begin(); 
              itr != _segmentList.end(); itr++)
    {
         (*itr).writeData(id);
    }
}

// Given a Route Attribute(PathContainer) for a route to be
// aggregated, combine the contents of the PathContainer with
// the contents of this container
void
TRIPPathContainer::aggregate(Sptr<TRIPRouteAttr> attr) 
                                                throw (TRIPBadDataException&)
{   
    cpLog(LOG_DEBUG, "TRIPPathContainer::aggregate");
    assert((attr->getType() == TRIP_ADVERTISEMENT_PATH) || 
    (attr->getType() == TRIP_ROUTED_PATH));
    Sptr<TRIPPathContainer> pathContainer;
    pathContainer.dynamicCast(attr);
    if (pathContainer.getPtr() != 0)
    {

        const PathSegmentList& pathList = pathContainer->getPathSegments();
        if (pathList.size() > 0) // adv path elements exist
        {
            if (_segmentList.size() == 0) // no elements exist in this Path Container
            {
                PathSegmentList::const_iterator itr;
                for (itr = pathList.begin(); itr != pathList.end(); itr++)
                {
                    addPathSegment(*itr); // add each segment to the this list
                }
            }
            else if (_segmentList.size() > 0) //this list contains elements 
            {         //need to do comparison
                // copy the the current list
                PathSegmentList tempSegList(getPathSegments());
                // clear the current list
                _segmentList.clear();
                _len = 0;

                TRIPItadPathSegment seqSeg(TRIP_SEG_AP_SEQ);
                TRIPItadPathSegment setSego(TRIP_SEG_AP_SET);
                TRIPItadPathSegment setSegt(TRIP_SEG_AP_SET);
                TRIPItadPathSegment setSegn(TRIP_SEG_AP_SET);

                // need to compare leading path seg in each container
                PathSegmentList::const_iterator thisItr;
                thisItr = tempSegList.begin();
                PathSegmentList::const_iterator otherItr;
                otherItr = pathList.begin();
                if (((*thisItr).getType() == TRIP_SEG_AP_SEQ) && 
                                ((*otherItr).getType() == TRIP_SEG_AP_SEQ))
                {
                    if ((*otherItr) == (*thisItr))
                    {
                        seqSeg = TRIPItadPathSegment((*thisItr));
                    }
                    else
                    {
                        list<u_int16_t> thisList = (*thisItr).getItadList();
                        list<u_int16_t> otherList = (*otherItr).getItadList();
                        list<u_int16_t>::iterator tItr, oItr;
                        tItr = thisList.begin();
                        oItr = otherList.begin();
                        bool stillMatching = true;
                        while((oItr != otherList.end()) || (tItr != thisList.end()))
                        {
                            if(stillMatching == true)
                            {
                                if (((*oItr) != (*tItr)) || (oItr == otherList.end()) || (tItr == thisList.end()))
                                {
                                    stillMatching = false;
                                }
                            }

                            if (stillMatching == true)
                            {
                                seqSeg.addItad((*tItr));
                                oItr++;
                                tItr++;
                            }
                            else 
                            {
                                if(oItr != otherList.end())
                                {
                                    setSego.addItad((*oItr));
                                    oItr++;
                                }
                                if(tItr != thisList.end())
                                {
                                    setSegt.addItad((*tItr));
                                    tItr++;
                                }
                            }
                        }
                    }
                }
                else // other is not AP_SEQ, this is not AP_SEQ, or both
                {
                    setSegt = TRIPItadPathSegment((*thisItr));
                    setSegt.setType(TRIP_SEG_AP_SET);
                    setSego = TRIPItadPathSegment((*otherItr));
                    setSego.setType(TRIP_SEG_AP_SET);
                }
                if (!seqSeg.isEmpty())
                {
                    addPathSegment(seqSeg);
                }

                for (thisItr = (tempSegList.begin())+1; thisItr != tempSegList.end(); thisItr++)
                {
                    list<u_int16_t> thisList = (*thisItr).getItadList();
                    list<u_int16_t>::iterator tItr;
                    for (tItr = thisList.begin(); tItr != thisList.end(); tItr++)
                    {
                        setSegt.addItad((*tItr));
                    }
                }

                if (!setSegt.isEmpty())
                {
                    addPathSegment(setSegt);
                }
                // checking if ITAD exists in setSegt


                if (!setSego.isEmpty())
                {
                    list<u_int16_t> thisList = setSego.getItadList();
                    list<u_int16_t>::iterator tItr;
                    for (tItr = thisList.begin(); tItr != thisList.end(); tItr++)
                    {
                        if (setSegt.containsItad((*tItr)) == false)
                        {
                            setSegn.addItad((*tItr));
                        }
                    }
                }
                for (otherItr = (pathList.begin())+1; otherItr != pathList.end(); otherItr++)
                {
                    list<u_int16_t> otherList = (*otherItr).getItadList();
                    list<u_int16_t>::iterator oItr;
                    for (oItr = otherList.begin(); oItr != otherList.end(); oItr++)
                    {
                        if (setSegt.containsItad((*oItr)) == false)
                        {
                            setSegn.addItad((*oItr));
                        }
                    }
                }
                if (!setSegn.isEmpty())
                {
                    addPathSegment(setSegn);
                }
                cpLog(LOG_DEBUG, "TRIPPathContainer::aggregate complete");


            }
        }
        else // adv Path of aggregating route is empty
        {
            cpLog(LOG_DEBUG, "TRIPPathContainer::aggregate other adv Path is empty");
        }
    }
    else
    {
        cpLog(LOG_DEBUG, "TRIPPathContainer::aggregate other adv Path does not exist");
        throw TRIPBadDataException("Bad TRIPPathContainer", __FILE__,
                                        __LINE__, TRIP_ERR_UPDATE);
    }

}


TRIPPathContainer::TRIPPathContainer(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    int hdrLen  = _hdr->getHdrLength(); 
    int len  = _hdr->getLength(); 

    _len = 0;
    parsePathSegments(buf+hdrLen, len);
}

void
TRIPPathContainer::parsePathSegments(const char* buf, u_int16_t len)
{
    u_int16_t tLen = 0;
    while(tLen < len)
    {
        TRIPItadPathSegment tsData(buf+tLen);                
        tLen += tsData.getLength();
        addPathSegment(tsData);
    }
}

void 
TRIPPathContainer::copyObj(const TRIPPathContainer& other)
{
    TRIPRouteAttr::copyObj(other);
    _segmentList.clear();
    _len = 0;
    PathSegmentList::const_iterator itr;
    for(itr = other._segmentList.begin(); 
              itr != other._segmentList.end(); itr++)
    {
         const TRIPItadPathSegment& data = (*itr);
         addPathSegment(data);
    }
}


bool 
TRIPPathContainer::isEqualAttribute(const TRIPRouteAttr& attr) const
{
    assert((attr.getType() == TRIP_ADVERTISEMENT_PATH) ||
           (attr.getType() == TRIP_ROUTED_PATH));

    const TRIPPathContainer* other = dynamic_cast<const TRIPPathContainer*>(&attr);
    assert(other);
    bool retVal = true;
    if(_segmentList.size() != other->_segmentList.size()) return false;
    PathSegmentList::const_iterator itr;
    PathSegmentList::const_iterator itr2;

    for(itr = _segmentList.begin(), itr2 = other->_segmentList.begin();
              itr != _segmentList.end(); itr++, itr2++)
    {
        const TRIPItadPathSegment& data = (*itr);
        const TRIPItadPathSegment& data2 = (*itr2);
        if(data == data2) 
        {
            retVal &= true;
        }
        else 
        {
            retVal &= false;
            return retVal;
        }
    }
    return retVal;
}

bool 
TRIPPathContainer::checkItadLoop(u_int16_t itad)
{
    for(PathSegmentList::const_iterator itr = _segmentList.begin(); 
              itr != _segmentList.end(); itr++)
    {
         if((*itr).containsItad(itad)) return true;
    }
    return false;
}
