/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
#include <stdio.h>

#include "TRIPRouteHdr.hxx"
#include "TRIPCoreDef.hxx"
#include "TRIPErrorHandler.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPRouteHdr::TRIPRouteHdr(TRIPAddrFamily addrFamily, TRIPAppProto appProtocol)
    : _addrFamily(addrFamily), _appProto(appProtocol)
{
    validate();
}

TRIPRouteHdr::TRIPRouteHdr(const char* buf) throw(TRIPBadDataException&)
{
    assert(buf);
    u_int16_t addrFam;
    u_int16_t appProto;
    memcpy(&addrFam, buf, 2);
    memcpy(&appProto, buf+2, 2);
    _addrFamily = static_cast<TRIPAddrFamily>(ntohs(addrFam)); 
    _appProto = static_cast<TRIPAppProto>(ntohs(appProto));
    validate();
}

void
TRIPRouteHdr::validate() throw (TRIPBadDataException&)
{
    if((_addrFamily < 1) || (_addrFamily >= TRIP_ADDR_MAX))
    {
        char mBuf[256];
        sprintf(mBuf, "BAD TRIP addrress family (%d)", 
                                                   _addrFamily);
        throw TRIPBadDataException(mBuf, __FILE__, __LINE__, 
                                        TRIP_ERR_UPDATE, U_MALFORMED_ATTR_LIST);
    }

    if((_appProto < 1) || (_appProto >= TRIP_APP_PROTO_MAX))
    {
        char mBuf[256];
        sprintf(mBuf, "BAD TRIP application protocol (%d)", _appProto);
        throw TRIPBadDataException(mBuf, __FILE__, __LINE__, 
                                        TRIP_ERR_UPDATE, U_MALFORMED_ATTR_LIST);
    }
}

TRIPRouteHdr::TRIPRouteHdr(const TRIPRouteHdr& other) 
{
    _addrFamily = other._addrFamily;
    _appProto = other._appProto;
}

TRIPRouteHdr& 
TRIPRouteHdr::operator=(const TRIPRouteHdr& other)
{
    if(this != &other)
    {
        _addrFamily = other._addrFamily;
        _appProto = other._appProto;
    }
    return (*this);
}

void 
TRIPRouteHdr::writeData(int id) const throw(VNetworkException&)
{
    u_int16_t aFam = _addrFamily;
    u_int16_t aProto = _appProto;
    aFam = htons(aFam);
    aProto = htons(aProto);
    TRIPUtil::writeData(id, &aFam, 2);
    TRIPUtil::writeData(id, &aProto, 2);
}
