/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <netinet/in.h>

#include "TRIPTransCapability.hxx"
#include "TRIPUtil.hxx"
#include "TRIPErrorHandler.hxx"
#include "cpLog.h"

static const int TRAN_CAP_MIN_LENGTH=4;

TRIPTransCapability::TRIPTransCapability(TRIPTransMode mode)
    : TRIPCapability(TRIP_CAP_SND_REC),
      _transMode(mode)
{
    _length = TRAN_CAP_MIN_LENGTH;
    validate();
};


TRIPTransCapability::TRIPTransCapability(const char* buf) 
            throw(TRIPBadDataException&)
    :TRIPCapability(buf)
{
    validate();
    assert(buf);
    if(_length != TRAN_CAP_MIN_LENGTH)
    {
        throw TRIPBadDataException("BAD (Send/Receive) capability data",
                                   __FILE__, __LINE__, TRIP_ERR_OPEN);
    }
    u_int32_t tMode;
    memcpy(&tMode, buf+4, TRAN_CAP_MIN_LENGTH);
    _transMode = static_cast<TRIPTransMode>(ntohl(tMode));
}

void
TRIPTransCapability::validate() const throw (TRIPBadDataException&)
{
    if(_code != TRIP_CAP_SND_REC)
        throw TRIPBadDataException("Invalid capability code",
                                      __FILE__, __LINE__, TRIP_ERR_OPEN);
}


TRIPTransCapability::TRIPTransCapability(const TRIPTransCapability& other) 
{
    copyObj(other);    
}

TRIPTransCapability& 
TRIPTransCapability::operator=(const TRIPTransCapability& other)
{
    if(this != &other)
    {
        copyObj(other);    
    }
    return (*this);
}

void
TRIPTransCapability::copyObj(const TRIPTransCapability& other)
{
    TRIPCapability::copyObj(other);
    _transMode = other._transMode;
}

void 
TRIPTransCapability::writeData(int id) const throw(VNetworkException&)
{
    TRIPCapability::writeData(id);
    u_int32_t tMode = _transMode;
    tMode = htonl(tMode);
    TRIPUtil::writeData(id, &tMode, 4);
}

void 
TRIPTransCapability::dump(ostream& os) const
{
    os.width(DUMP_LABEL_WIDTH);
    os << "(Send/Receive)Capability:" ;
    os << static_cast<u_int32_t>(_transMode) << endl;
}

