#ifndef TRIPUpdateMsg_hxx
#define TRIPUpdateMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TRIPUpdateMsg_hxx_Version = 
"$Id: TRIPUpdateMsg.hxx,v 1.20 2001/02/16 00:51:05 sprajpat Exp $";

#include <set>

#include "Sptr.hxx"
#include "TRIPMsg.hxx"
#include "TRIPId.hxx"
#include "TRIPWithdrawnRoute.hxx"
#include "TRIPReachableRoute.hxx"
#include "TRIPAdvertisePath.hxx"
#include "TRIPRoutedPath.hxx"
#include "TRIPNextHopServer.hxx"
#include "TRIPAtomicAggregate.hxx"
#include "TRIPLocalPref.hxx"
#include "TRIPCommunity.hxx"
#include "TRIPMultiExitDisc.hxx"
#include "TRIPItadTopology.hxx"
#include "TRIPAuthAttr.hxx"


/** Implements a TRIP Update message */
class TRIPUpdateMsg : public TRIPMsg
{
    public:
       ///
       typedef set<Sptr<TRIPRouteAttr> >  AttrSet;

       ///
       TRIPUpdateMsg();

       ///
       TRIPUpdateMsg(Sptr<TRIPWithdrawnRoute> wRoute,
                    Sptr<TRIPReachableRoute> rRoute, const AttrSet& aSet,
                    bool isExternal=false);

       ///
       bool isAUpdateMsg() const { return true; }; 

       ///
       TRIPOpCode getType() const  { return TRIP_OP_UPDATE; };

       ///
       virtual void dump(ostream& os)const;


       ///Writes object to given network socket in network byte order
       void writeData(int id) const throw(VNetworkException&);

       ///
       string className() const { return "TRIPUpdateMsg"; };

       ///Checks the sanity of TRIP Msg, and throws a Bad Data exception
       void checkSanity() const throw (TRIPBadDataException&);

       /**@name Accessor */
       //@{
       ///
       const Sptr<TRIPWithdrawnRoute>& getWithdrawnRoute() const 
       { 
           return _withdrawnRoute; 
       }
       ///
       const Sptr<TRIPReachableRoute>& getReachableRoute() const 
       { 
           return _reachableRoute; 
       }

       ///
       bool hasAtomicAggregate() const ;

       ///
       Sptr<TRIPRouteAttr> getAttribute(TRIPAttrType aType) const;

       ///
       bool hasLocalPref() const ;

       ///
       bool hasCommunity() const ;

       ///
       bool hasMultiExitDisc() const ;

       ///
       bool hasItadTopology() const ;

       ///
       bool hasAuthAttr() const ;

       ///Set of attributes except Reachable/Withdrawn attribute types
       const set<Sptr<TRIPRouteAttr> >& getAttributes() const;

       ///
       u_int32_t getTransactionId() const { return _transactionId; };

       //@}

       ///Add Attribute to the message
       void addAttribute(Sptr<TRIPRouteAttr> attr);

       ///
       void addWithdrawnRoute(Sptr<TRIPWithdrawnRoute> route);

       ///
       void addReachableRoute(Sptr<TRIPReachableRoute> route);


       /**Validates a UPDATE message, return false when validation fails and
        * sets the eCode , subCode and Message string.
        */
       bool validate(u_int16_t itad, u_int8_t &eCode, u_int8_t& subCode, string& msg) const;

       ///Check if ReachableRoute, WithdrawnRoute and ITAD Topology attr are link state attribute
       bool checkLinkStateAttributes(u_int8_t &eCode, u_int8_t& subCode, string& msg);

       ///
       bool extCheckInvalidAttributes(u_int8_t &eCode, u_int8_t& subCode, string& msg);


       /** Checks the attribute flags of each attribute in UPDATE message
        *  return false when check fails.The msg is set to the (type, length
        *  and value) of the failed attribute.
        */
       bool checkAttrFlags(string& msg) const;

       /** Checks the attribute length of each attribute in UPDATE message
        *  return false when check fails.The msg is set to the (type, length
        *  and value) of the failed attribute.
        */
       bool checkAttrLengths(string& msg) const;

    private:
       ///
       void validateDuplicateAttr(u_int16_t type)
               throw (TRIPBadDataException&);

       ///
       void setTransactionId(u_int32_t trId) { _transactionId = trId; };

       ///
       void parse(const TRIPHeader& hdr,
                          const char* data) throw (TRIPBadDataException&);

       ///
       void parse(const char* data) throw (TRIPBadDataException&);

       /**@name Mandatory attributes */
       //@{

       ///
       Sptr<TRIPWithdrawnRoute>    _withdrawnRoute;
       ///
       Sptr<TRIPReachableRoute>    _reachableRoute;

       //@}

       ///Set of attributes except Reachable/Withdrawn
       AttrSet                    _attrSet;
   
       ///Uniquely identifies a Update message entering LS
       u_int32_t                  _transactionId;

       ///Set of flags for Update msg Attributes
       fd_set  _aFlagSet;

       ///
       friend class TRIPMsgParser;
       ///
       friend class LSPeerSession;
};

#endif
