/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CopyOnWriteData_cxx_Version =
    "$Id: CopyOnWriteData.cxx,v 1.19 2001/03/15 20:18:30 bko Exp $";


//Authors: Sunitha Kumar, Cullen Jennings

#include "CopyOnWriteData.hxx"
#include <cstdio>
#include <ctype.h>
#include "cpLog.h"

#if USE_HASH_MAP
#include <hash_map>
#endif

#define INLINE_ inline

static const int baseMultiple = 32;

INLINE_
CopyOnWriteData::CopyOnWriteData()
        :
        data_()
{}


INLINE_
CopyOnWriteData::CopyOnWriteData( const char* str, int length )
        :
        data_(str, length)
{}


INLINE_
CopyOnWriteData::CopyOnWriteData( const char* str )
        :
        data_(str, mystrlen(str))
{}


INLINE_
CopyOnWriteData::CopyOnWriteData( const string& str )
        :
        data_(str.c_str(), str.length())
{}


INLINE_
CopyOnWriteData::CopyOnWriteData( const mstring& mstr )
        :
        data_(mstr.c_str(), mstr.length())
{}


INLINE_
CopyOnWriteData::CopyOnWriteData( int value )
        :
        data_(8)
{
    char str[256];
    sprintf(str, "%d", value);
    int tmp = mystrlen(str);
    data_.makeWritable(tmp);
    memcpy(data_.bitsPtr, str, tmp);
    data_.size = tmp;
}


INLINE_
CopyOnWriteData::CopyOnWriteData( const CopyOnWriteData& data )
        :
        data_(data.data_)
{}


INLINE_
CopyOnWriteData&
CopyOnWriteData::operator=( const char* str )
{
    int tmp = mystrlen(str);

    data_.clear();
    data_.makeWritable(tmp);
    memcpy(data_.bitsPtr, str, tmp);
    data_.size = tmp;
    return *this;
}


INLINE_
CopyOnWriteData&
CopyOnWriteData::operator=( const CopyOnWriteData& data )
{
    if (this != &data)
    {
        data_.clear();
        data_ = data.data_;
    }
    return *this;
}


INLINE_
const char*
CopyOnWriteData::getData() const
{
    data_.makeCString();
    return data_.bitsPtr;
}


INLINE_
const char*
CopyOnWriteData::getDataBuf() const
{
    return data_.bitsPtr;
}


INLINE_
char
CopyOnWriteData::getChar(int i) const
{
    assert(i < data_.size);
    return data_.bitsPtr[i];
}


INLINE_
void
CopyOnWriteData::setchar(int i, char c)
{
    data_.makeWritable(i + 1);

    if (i >= data_.size)
    {
        // do something now
        data_.size = i + 1;
    }
    data_.bitsPtr[i] = c;
}


INLINE_
char
CopyOnWriteData::operator[](int i) const
{
    return getChar(i);
}


INLINE_
int
CopyOnWriteData::length() const
{
    return data_.size;
}


INLINE_
int
CopyOnWriteData::compare(const char* str, int length) const
{
    return data_.compare(str, length);
}


INLINE_
int
CopyOnWriteData::compare(const CopyOnWriteData& data) const
{
    return data_.compare(data.data_.bitsPtr, data.data_.size);
}

INLINE_
int
CopyOnWriteData::compareNoCase(const char* str, int length) const
{
    return data_.compareNoCase(str, length);
}


INLINE_
int
CopyOnWriteData::compareNoCase(const CopyOnWriteData& data) const
{
    return data_.compareNoCase(data.data_.bitsPtr, data.data_.size);
}


INLINE_
bool
CopyOnWriteData::operator==(const char* str) const
{
    return ( data_.compare(str, mystrlen(str)) == 0 );
}


INLINE_
bool
CopyOnWriteData::operator==( const CopyOnWriteData& data) const
{
    return ( data_.compare(data.data_.bitsPtr, data.data_.size) == 0 );
}


INLINE_
bool
CopyOnWriteData::operator!=(const char* str) const
{
    return !operator==(str);
}


INLINE_
bool
CopyOnWriteData::operator!=(const CopyOnWriteData& data) const
{
    return !operator==(data);
}


INLINE_
bool
CopyOnWriteData::operator>(const CopyOnWriteData& data) const
{
    return ( data_.compare(data.data_.bitsPtr, data.data_.size) > 0 );
}


INLINE_
bool
CopyOnWriteData::operator<(const CopyOnWriteData& data) const
{
    return ( data_.compare(data.data_.bitsPtr, data.data_.size) < 0 );
}


INLINE_
CopyOnWriteData
CopyOnWriteData::operator+(const CopyOnWriteData& data) const
{
    CopyOnWriteData tmp(*this);
    tmp += data;
    return tmp;
}


INLINE_
CopyOnWriteData
CopyOnWriteData::operator+(const char* str) const
{
    CopyOnWriteData tmp(*this);
    tmp += str;
    return tmp;
}


INLINE_
void
CopyOnWriteData::operator+=(const CopyOnWriteData& data)
{
    if (data.length())
    {
        data_.makeWritable(length() + data.length());
        memcpy(data_.bitsPtr + data_.size,
               data.data_.bitsPtr, data.length());
        data_.size += data.length();
    }
}


INLINE_
void
CopyOnWriteData::operator+=(const char* str)
{
    int len = mystrlen(str);
    if (len)
    {
        data_.makeWritable(length() + len);
        memcpy(data_.bitsPtr + data_.size, str, len);
        data_.size += len;
    }
}


INLINE_
CopyOnWriteData::~CopyOnWriteData()
{}



INLINE_
void
CopyOnWriteData::erase()
{
    data_.clear();
}


INLINE_
CopyOnWriteData::operator string() const
{
    return string(data_.bitsPtr, static_cast < unsigned int > (length()));
}


INLINE_
CopyOnWriteData::operator const char*() const
{
    return getData();
}


INLINE_
CopyOnWriteData::operator mstring() const
{
    return mstring(getData());
}


INLINE_
CopyOnWriteData::operator int() const
{
    int l = data_.size;
    int val = 0;
    char* p = data_.bitsPtr;

    while (l--)
    {
        char c = *p++;
        if ((c >= '0') && (c <= '9'))
        {
            val *= 10;
            val += c - '0';
        }
        else
        {
            return val;
        }
    }
    return val;
}


INLINE_
void CopyOnWriteData::tolower()
{
    int len = length();

    data_.makeWritable(len);
    int i = 0;
    while(i < len)
    {
	data_.bitsPtr[i] = std::tolower(data_.bitsPtr[i]);
	i++;
    }
}


INLINE_
void CopyOnWriteData::toupper()
{
    int len = length();

    data_.makeWritable(len);
    int i = 0;
    while(i < len)
    {
	data_.bitsPtr[i] = std::toupper(data_.bitsPtr[i]);
	i++;
    }
}


INLINE_
string CopyOnWriteData::getstring() const
{
    return string(data_.bitsPtr, static_cast < unsigned int > (length()));
}


INLINE_
int
CopyOnWriteData::find( const CopyOnWriteData& match, int start )
{
    int pos = 0;
    if ( start != npos )
    {
        pos = start;
    }

    const char* ptr = data_.bitsPtr + pos;
    while ((pos + match.data_.size) <= data_.size)
    {
        if ( (*(ptr) == *match.data_.bitsPtr ) &&
                (strncmp(ptr,
                         match.data_.bitsPtr,
                         match.data_.size)
                 == 0))
        {
            // a match has been found
            return pos;
        }
        ++ptr;
        ++pos;  // or not
    }

    return npos;
}


INLINE_
int
CopyOnWriteData::find( const char* match, int start )
{
    int matchLength = mystrlen(match);

    int pos = 0;
    if ( start != npos )
    {
        pos = start;
    }

    int compares = data_.size - matchLength - pos;
    const char* ptr = data_.bitsPtr + pos;
    while ((pos + matchLength) <= data_.size)
        //    while((pos + matchLength) <= data_.size)
        while (compares-- >= 0)
        {
            if (
                (*match == *(ptr)) &&
                ( strncmp(ptr,
                          match,
                          matchLength)
                  == 0))
            {
                // a match has been found
                return pos;
            }
            ++ptr;
            ++pos;  // or not
        }

    return npos;
}


INLINE_
int
CopyOnWriteData::match( const char* match,
                        CopyOnWriteData* retModifiedCopyOnWriteData,
                        bool doReplace,
                        CopyOnWriteData replaceWith)
{
    if (replaceWith != "")
    {
        int retVal;
        string::size_type pos = getstring().find(match);

        if (pos == string::npos)
        {
            return NOT_FOUND;
        }

        int matchLength = mystrlen(match);

        string::size_type replacePos = pos + matchLength;
        retVal = FIRST;

        if (retModifiedCopyOnWriteData)
        {
            (*retModifiedCopyOnWriteData) = getstring().substr(0, pos);
            if (retModifiedCopyOnWriteData->length())
            {
                retVal = FOUND;
            }
        }

        if (doReplace)
        {
            if (replacePos <= static_cast < unsigned int > (length()) )
            {
                replace(0, replacePos, replaceWith);
            }
            else
            {
                printf("pos=%d  match.len=%d replacePos=%d  buf.size()= %d\n",
                       pos,
                       matchLength,
                       replacePos,
                       length() );

            }
        }

        return retVal;
    }
    else
    {
        int retVal;
        int pos = find(match);

        if (pos == npos)
        {
            return NOT_FOUND;
        }

        int matchLength = mystrlen(match);

        int replacePos = pos + matchLength;
        retVal = FIRST;

        if (retModifiedCopyOnWriteData)
        {
            (*retModifiedCopyOnWriteData) = getstring().substr(0, pos);
            if (retModifiedCopyOnWriteData->length())
            {
                retVal = FOUND;
            }
        }

        if (doReplace)
        {
            if (replacePos <= length() )
            {
                data_.truncate(replacePos, 0);
            }
            else
            {
                printf("pos=%d  match.len=%d replacePos=%d  buf.size()= %d\n",
                       pos,
                       matchLength,
                       replacePos,
                       length() );
            }
        }

        return retVal;
    }
}


INLINE_
bool isIn(char c, const char* match)
{
    char p;

    while((p = *match++))
    {
	if(p == c)
	{
	    return true;
	}
    }

    return false;
}


INLINE_
CopyOnWriteData
CopyOnWriteData::parseOutsideQuotes( const char* match, 
                                     bool useQuote,
                                     bool useAngle,
                                     bool* matchFail )
{
    int pos = 0;

    bool inDoubleQuotes = false;
    bool inAngleBrackets = false;

    bool foundAny = false;

    while(!foundAny && (pos < data_.size))
    {
	char p = *(data_.bitsPtr + pos);

        switch (p)
        {
        case '"':
            if(!inAngleBrackets && useQuote)
            {
                inDoubleQuotes = !inDoubleQuotes;
            }
            break;
        case '<':
            if(!inDoubleQuotes && useAngle)
            {
                inAngleBrackets = true;
            }
            break;
        case '>':
            if(!inDoubleQuotes && useAngle)
            {
                inAngleBrackets = false;
            }
            break;
        default:
            break;
        }

	if(!inDoubleQuotes && !inAngleBrackets && isIn(p, match))
	{
	    foundAny = true;
	}
	pos++;
    }

    int pos2 = pos;

    while(
	foundAny && 
	(pos2 < data_.size) && 
	isIn(*(data_.bitsPtr + pos2), match)
	)
    {
	pos2++;
    }

    CopyOnWriteData result;

    if(foundAny)
    {
	result = *this;
	result.data_.substr(0, pos - 1);

	data_.substr(pos2, length());
	if(matchFail)
	{
	    *matchFail = false;
	}
    }
    else
    {
	if(matchFail)
	{
	    *matchFail = true;
	}
    }

    return result;
}


INLINE_
CopyOnWriteData
CopyOnWriteData::parse( const char* match, bool* matchFail )
{
    int pos = 0;

    bool foundAny = false;

    while(!foundAny && (pos < data_.size))
    {
	char p = *(data_.bitsPtr + pos);
	if(isIn(p, match))
	{
	    foundAny = true;
	}
	pos++;
    }

    int pos2 = pos;

    while(
	foundAny && 
	(pos2 < data_.size) && 
	isIn(*(data_.bitsPtr + pos2), match)
	)
    {
	pos2++;
    }

    CopyOnWriteData result;

    if(foundAny)
    {
	result = *this;
	result.data_.substr(0, pos - 1);

	data_.substr(pos2, length());
	if(matchFail)
	{
	    *matchFail = false;
	}
    }
    else
    {
	if(matchFail)
	{
	    *matchFail = true;
	}
    }

    return result;
}


INLINE_
CopyOnWriteData
CopyOnWriteData::matchChar( const char* match, char* matchedChar )
{
    int pos = 0;

    bool foundAny = false;

    while(!foundAny && (pos < data_.size))
    {
	char p = *(data_.bitsPtr + pos);
	if(isIn(p, match))
	{
	    foundAny = true;
	    if(matchedChar)
	    {
		*matchedChar = p;
	    }
	}
	pos++;
    }

    CopyOnWriteData result;

    if(foundAny)
    {
	result = *this;
	result.data_.substr(0, pos - 1);
	data_.substr(pos, length());
    }
    else if(matchedChar)
    {
	*matchedChar = '\0';
    }

    return result;
}


INLINE_
CopyOnWriteData
CopyOnWriteData::getLine( bool* matchFail )
{
    const int STARTING = 0;
    const int HAS_CR = 1;
    const int HAS_LF = 2;
    const int HAS_CRLF = 3;

    int state = STARTING;
    int pos = 0;

    bool foundAny = false;

    while(!foundAny && (pos < data_.size))
    {
	char p = *(data_.bitsPtr + pos);
	if( p == '\r' )
	{
	    state = HAS_CR;
	}
	else if (p == '\n' )
	{
	    if(state == HAS_CR)
	    {
		state = HAS_CRLF;
	    }
	    else
	    {
		state = HAS_LF;
	    }
	    foundAny = true;
	}
	else
	{
	    state = STARTING;
	}
	pos++;
    }

    int pos2 = pos;

    if(state == HAS_CRLF)
    {
	pos--;
    }

    CopyOnWriteData result;

    if(foundAny)
    {
	result = *this;
	result.data_.substr(0, pos - 1);

	data_.substr(pos2, length());
	if(matchFail)
	{
	    *matchFail = false;
	}
    }
    else
    {
	if(matchFail)
	{
	    *matchFail = true;
	}
    }

    return result;
}


INLINE_
bool isEqualNoCase(const CopyOnWriteData& left, const CopyOnWriteData& right)
{
    return (left.compareNoCase(right) == 0);
}


INLINE_
bool isEqualNoCase(const char* left, const CopyOnWriteData& right)
{
    return (right.compareNoCase(left, mystrlen(left)) == 0);
}


INLINE_
void
CopyOnWriteData::replace(int startpos, int endpos,
                         const CopyOnWriteData& replaceStr)
{
    if (length())
    {
        string str = getstring();

        str.replace(startpos, endpos, replaceStr.getstring());
        *this = CopyOnWriteData(str);
    }
}


INLINE_
void CopyOnWriteData::removeSpaces()
{
   if (length() == 0)
    {
        return ;
    }
 
    int firstChar = 0;
 
    while ((firstChar < length()) && (getChar(firstChar) == ' '))
    {
        firstChar++;
    }
 
    int lastChar = length();
    while ((lastChar > 0) && (getChar(lastChar - 1) == ' '))
    {
        lastChar--;
    }
    if (firstChar > lastChar)
    {
        firstChar = lastChar;
    }                      
    data_.substr(firstChar, lastChar);       
}


INLINE_
void CopyOnWriteData::removeLWS()
{
    int replaceTo;
    int pos = -1;
    pos = find ("\r\n");
    if (pos > length())
    {
        pos = find("\n");
    }
    else
    {
      replaceTo = pos+1; //should end after \r\n
    }
    if (pos > length())
    {
	return;
    }
    else
    {
      replaceTo = pos; //should end after \n
    }
    bool replaceFlag = false;
    do
    {
        while ( (replaceTo+1 < length()) && 
                ( (getChar(replaceTo+1) == '\t') ||
                  (getChar(replaceTo+1) == ' ')
                ) 
              )
        {
	    char temp = getChar(replaceTo);
            replaceTo++;
	    temp = getChar(replaceTo);
            replaceFlag = true;
        }
        if (replaceFlag)
        {
            int replaceFrom = pos;
            while ( (replaceFrom-1 > 0) &&
                    ( (getChar(replaceFrom-1) == '\t') ||
                      (getChar(replaceFrom-1) == ' ')
                    )
                  )
            {
	        char temp = getChar(replaceFrom);
                replaceFrom--;
		temp = getChar(replaceFrom);
            }
            int replaceNum = replaceTo-replaceFrom;
            replace(replaceFrom, replaceNum, ""); //replace with nothing
	   
        }
        //remember pos.
        int initpos = pos;
        pos = find("\r\n", initpos+2);
	if (pos > length())
	{
	    pos = find("\n", initpos+2);
	    replaceTo = pos;
        }
	else
        {
	    replaceTo = pos+1;
        }
    }
    while (pos < length());
}


INLINE_
void
CopyOnWriteData::expand(CopyOnWriteData startFrom,
                        CopyOnWriteData findstr,
                        CopyOnWriteData replstr,
                        CopyOnWriteData delimiter)
{
    int startPos = find(startFrom);
    if (startPos < npos)
    {
        int delimPos
        = find(delimiter, startPos);
        int findPos
        = find(findstr, startPos);

        while (findPos < delimPos)
        {
            //found replstr, replace
            replace( findPos, findstr.length(), replstr.getstring());
            //find next.
            delimPos
		= find( delimiter,
			findPos + replstr.length());
            findPos
		= find( findstr, findPos);
        }
    }
}


INLINE_
ostream&
operator<<(ostream& s, const CopyOnWriteData& data)
{
    s.write(data.data_.bitsPtr, data.data_.size);
    return s;
}


INLINE_
bool
operator==(const char* str, const CopyOnWriteData& d)
{
    return ( d.data_.compare(str, mystrlen(str)) == 0 );
}

#if 0
INLINE_
bool
operator!=(const char* str, CopyOnWriteData& d)
{
    return (d.data_.compare(str, mystrlen(str)) != 0 );
}
#endif

/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
