#ifndef VRWLock_hxx
#define VRWLock_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VRWLock_hxx_Version =
    "$Id: VRWLock.hxx,v 1.6 2001/03/09 08:30:46 icahoon Exp $";

#include "VMutex.h"
#include "VCondition.h"

/** Read/write lock. Allows serialized write access and concurrent
 *  read access.
 */
class VRWLock
{
    public:

        /** Create a lock.
         */
        VRWLock();


        /** Destroy the lock.
         */        
        ~VRWLock();
        
        
        /** Obtain the read lock. This will block until all writers have
         *  completed.
         */ 
        void ReadLock();
        

        /** Obtain the write lock. Provides serialized writing.
         */
        void WriteLock();
        

        /** Unlocks either the read or write lock.
         */
        void Unlock();


    private:


        /** Access to the lock is synchronous.
         */
        VMutex      myMutex;


        /** Readers will block until all writers have completed.
         */
        VCondition  myReadCondition;

        
        /** Pending writers will block while while waiting for the writer
         *  to finish.
         */
        VCondition  myPendingWriteCondition;
        

        /** We can have multiple readers blocking.
         */
        u_int32_t   myReaderCount;
        

        /** We can have only a single writer.
         */
        bool        myWriterHasLock;


        /** We can have many pending writers.
         */
        u_int32_t   myPendingWriterCount;
};

#endif
