/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const DataVerify_cxx_Version =
    "$Id: DataVerify.cxx,v 1.2 2001/03/15 20:18:31 bko Exp $";

#include "Data.hxx"
#include "Verify.hxx"

void test()
{
    {
	Data xyz("AllCaps");
	
	xyz.toupper();
	test_verify(xyz == "ALLCAPS");
	
	xyz.tolower();
	test_verify(xyz == "allcaps");
    }

    {
	Data xyz("allcaps");
	test_verify("allcaps" == xyz);
	test_verify("allCaps" != xyz);
    }

    /// test parseOutsideQuotes

    {
	Data test("\"Ogawa, Bryan\" <sip:bko@cisco.com,foo,bar,city>,\"Doe, John\" <sip:johnd@test.com,,>,Test Person <sip:test@test.com>");

	bool fail;
	Data a = test.parseOutsideQuotes(",", true, true, &fail);
	Data b = test.parseOutsideQuotes(",", true, true, &fail);
	test_verify(a == "\"Ogawa, Bryan\" <sip:bko@cisco.com,foo,bar,city>");
	test_verify(b == "\"Doe, John\" <sip:johnd@test.com,,>");
	test_verify(test == "Test Person <sip:test@test.com>");
	test_verify(fail == false);
    }

    {
	Data test("\",,,\" <,,,,>,\",,\" <,,>,\",,,\"<,,,>");

	bool fail;
	Data a = test.parseOutsideQuotes(",", true, false, &fail);
	test_verify(a == "\",,,\" <");

	test = "<,,,,> \",,\" <,,>,\",,,\"<,,,>";
	Data b = test.parseOutsideQuotes(",", false, true, &fail);
	test_verify(b == "<,,,,> \"");

	test = "<<,<,<,,,<,<,<,   ,,,, , ,  >,>  ";
	Data c = test.parseOutsideQuotes(",", false, true, &fail);
	test_verify(c == "<<,<,<,,,<,<,<,   ,,,, , ,  >");
    }

    {
	Data test;

	test = Data("a") + Data("b");
	test_verify(test == "ab");

	Data second = "Ogawa";

	test = "Bryan " + second;
	test_bug(test == "Bryan Ogawa");

        Data number = "7";    
        Data result;
            
        result = "Wigets: " + number;
	test_bug(result == "Wigets: 7");
    }

}


int main()
{
    test();
    return test_return_code(12, 2);
}
